<?php

namespace HCM\AVL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;

class NivelAvaliacaoController extends Crud {
    protected $dataSourceName = "";
    protected $dataSourceName1 = 'avl_nivelavaliacao';
    protected $dataSourceNameGet = 'getNivelAvaliacao';
    protected $dataSourceNameQuestao = 'avl_nivelavaliacaoquestao';
    protected $dataSourceNameGetQuestao = 'getNivelAvaliacaoQuestao';
    protected $dataSourceNameGetSelect = 'getNivelAvaliacaoQuestaoSelect';

    
    protected $NivelAvaliacaoService;

     protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRNIVELAVALIA');
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\AVL\Services\NivelAvaliacaoService $nivelAvaliacaoService){
        $this->NivelAvaliacaoService = $nivelAvaliacaoService;
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getNivelAvaliacao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setNivelAvaliacao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceName1;
            foreach($request->getDataSet()->getRows() as $row) {
                if(array_key_exists('NRNIVELAVALIA', $row)){
                    if($row['__is_new'])
                        $row['NRNIVELAVALIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'NIVELAVALIACAO', 1, 22, true);
                }else{
                    $row['NRNIVELAVALIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'NIVELAVALIACAO', 1, 22, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    public function deleteNivelAvaliacao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceName1;
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $questoes = $this->NivelAvaliacaoService->getQuestaoAvaliacao($dataset);
            if(empty($questoes))
                parent::delete($request, $response);
            else
                $response->addMessage(new Message("Existem questões reacionadas a este nivel de avaliacão, favor exclui-las", Message::TYPE_WARNING));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    public function getNivelAvaliacaoQuestao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGetQuestao);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function deleteQuestaoPorNivelAval(DTO\Request\DataSet $request, DTO\Response $response){
       try{
           $this->dataSourceName = $this->dataSourceNameQuestao;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
        
    }
    
    public function setQuestaoPorNivelAval(DTO\Request\DataSet $request, DTO\Response $response){
        try{
           $this->dataSourceName = $this->dataSourceNameQuestao;
            foreach($request->getDataSet()->getRows() as $row) {
                if(array_key_exists('NRNIVELQUESTAVA', $row)){
                    if($row['__is_new'])
                        $row['NRNIVELQUESTAVA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'NIVELQUESTAVAL', 1, 22, true);
                }else{
                    $row['NRNIVELQUESTAVA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'NIVELQUESTAVAL', 1, 22, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    
    public function getNivelAvaliacaoQuestaoSelect(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGetSelect);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
}