<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\FPA\Services\RelatCustomDirfService;
use HCM\Util\HCMParameters;

class RelatCustomDirfController extends Crud {

    CONST operatorLikeAll = 'LIKE_ALL';
    CONST operatorLike = 'LIKE';

    public function __construct (Manager $dataSourceManager, HCMParameters $hcmParameters, RelatCustomDirfService $relatCustomDirfService){
        $this->relatCustomDirfService = $relatCustomDirfService;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getRelatCustomDirf(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            set_time_limit(0);
            ini_set('memory_limit', '2048M');
            $filters = \HCM\Util\Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());
            $parameters = array();
            $fieldsUnset = array('P_DTMESCOMPETENC', 'P_NRORG', 'NRTPMODALIDCAL', 'NRESTRUTLEGAL', 'NRESTRUTGEREN', 'IDDADOSVINC', 'NRGRPDIRF');
            foreach ($filters as $key => $filter) {
                if (array_search($filter['columnName'], $fieldsUnset) !== false){
                    if($filter['columnName'] === 'NROCORRECAL'){
                        $filter['value'] = str_replace("%", "", $filter['value']);
                    }
                    unset($filters[$key]);
                }
                if($filter['operator'] == self::operatorLikeAll){
                    $filters[0]['operator'] = self::operatorLike;
                    $filters[0]['columnName'] = str_replace('|', '||', $filter['columnName']);
                }
                $parameters[$filter['columnName']] = $filter['value'];
            }
            $parameters['NRTPMODALIDCAL'] = $this->hcmParameters->getTipoModalidadeCalculo();
            $result = $this->relatCustomDirfService->getRelatCustomDirf($parameters, $filters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getRelatCustomDirf', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }
} 