<?php

namespace HCM\FPA\ObrigacaoLegal\CNAB\Banestes;

use HCM\Util\FPA;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\Util;
use HCM\FPA\Entity\FpaFormcnabEntity;
use HCM\FPA\Entity\FpaCnabmagnetEntity;

class Cnab021 {
    const newLine = '<br>';
    
    private $headerResult;  //Resultados de query, e de dados fixos (formulárionão formatado
    private $bodyResult;
    private $traillerResult;

    private $headerString   = array(); //String de saída para arquivo txt já formatada
    private $bodyString     = array();
    private $traillerString = array();

    private $nrSequencial;  //Sequencial dos registros
    private $vrTotal;       //Soma total dos valores

    private $queryHeader;   //Query's completas com where e order by
    private $queryBody;

    private $nrForm;        //Dados do formulário
    private $nrOrg;
    private $nrEvento;
    private $competencia;
    private $tpMovimento;
    private $modalidCalc;
    private $ocorrenCalc;
    private $ordem; //Escopo selecionado. Ordenamento que o dsIni e dsFim deverão considerar
    private $dsIni; //Inicio e fim do escopo selecionado
    private $dsFim;
    private $tpVinculo;
    private $idUtilDtPgtoItem;
    private $idIgnoraOcorrencia;
    private $dtPagamento;
    private $nrEstrutResp;
    private $idGeraMagnet;
    private $cdBanco;
    private $idTeste;
    private $cdOperador;
    private $idatualizacad;
    private $cdConvenio;

    //Arrays de configuração de layouts
    private $layoutHeader = array(  'TIPOREGIS' => array( 'tipo' => 'N', 'ini'  =>   1, 'fim'  =>   1 ),    /*001 001 Tipo de registro = 1*/
                                    'CDCONTRAT' => array( 'tipo' => 'N', 'ini'  =>   2, 'fim'  =>   6 ),    /*002 006 Código do contratante*/
                                    'NMCONTRAT' => array( 'tipo' => 'A', 'ini'  =>   7, 'fim'  =>  36 ),    /*007 036 Nome do contratante*/
                                    'CNPJCONTR' => array( 'tipo' => 'N', 'ini'  =>  37, 'fim'  =>  50 ),    /*037 050 CNPJ do contratante*/
                                    'CDCCCONTR' => array( 'tipo' => 'N', 'ini'  =>  51, 'fim'  =>  61 ),    /*051 061 Conta corrente do contratante*/
                                    'DTGRAVACA' => array( 'tipo' => 'Y'/*'ini'  =>  62, 'fim'  => 69*/),    /*062 069 Data de Gravação do Arquivo AAAAMMDD*/
                                    'TIPOREMES' => array( 'tipo' => 'N', 'ini'  =>  70, 'fim'  =>  70 ),    /*070 070 Tipo de remessa 1 pagamento 7 cadastro*/
                                    'ORIGEMARQ' => array( 'tipo' => 'A', 'ini'  =>  71, 'fim'  =>  73 ),    /*071 073 Origem do arquivo = FP3*/
                                    'REFPAGAME' => array( 'tipo' => 'N', 'ini'  =>  74, 'fim'  =>  79 ),    /*074 079 Referencia do pagamento*/
                                    'DTPAGAMEN' => array( 'tipo' => 'Y'/*'ini'  =>  80, 'fim'  => 87*/),    /*080 087 Data de crédito*/
                                    'TABELAERR' => array( 'tipo' => 'A', 'ini'  =>  88, 'fim'  => 147 ),    /*088 147 uso do banco, numérico, tabela de erros*/
                                    'BRANCOS01' => array( 'tipo' => 'A', 'ini'  => 148, 'fim'  => 506 ),    /*148 506 brancos*/
                                    'NRSEQUREG' => array( 'tipo' => 'N', 'ini'  => 507, 'fim'  => 512 ) );  /*507 512 Sequência do registro = 000001*/
    private $layoutBody = array(    'TIPOREGIS' => array( 'tipo' => 'N', 'ini'  =>   1, 'fim'  =>   1 ),    /*001 001 tipo de registro = 2*/
                                    'CDMATRICU' => array( 'tipo' => 'N', 'ini'  =>   2, 'fim'  =>  10 ),    /*002 010 matrícula do empregado*/
                                    'NMEMPREGA' => array( 'tipo' => 'A', 'ini'  =>  11, 'fim'  => 110 ),    /*011 110 Nome do empregado*/
                                    'CDFORMPAG' => array( 'tipo' => 'N', 'ini'  => 111, 'fim'  => 111 ),    /*111 111 forma de pagamento 3 ou 6 - USANDO 3*/
                                    'CDAGENCIA' => array( 'tipo' => 'N', 'ini'  => 112, 'fim'  => 115 ),    /*112 115 Código da agência da conta do beneficiário*/
                                    'CDCONTACO' => array( 'tipo' => 'N', 'ini'  => 116, 'fim'  => 126 ),    /*116 126 Número da conta corrente*/
                                    'CDCPFEMPR' => array( 'tipo' => 'N', 'ini'  => 127, 'fim'  => 140 ),    /*127 140 CPF do empregado*/
                                    'VRPAGAMEN' => array( 'tipo' => 'N', 'ini'  => 141, 'fim'  => 151 ),    /*141 151 Valor do pagamento*/
                                    'TPDOCUMEN' => array( 'tipo' => 'N', 'ini'  => 152, 'fim'  => 152 ),    /*152 152 Tipo de documento de identificação 1 CI ou 5 CTPS/MT - USANDO 1*/
                                    'IDENTIFIC' => array( 'tipo' => 'A', 'ini'  => 153, 'fim'  => 167 ),    /*153 167 Identificação*/
                                    'ORGEMISID' => array( 'tipo' => 'A', 'ini'  => 168, 'fim'  => 177 ),    /*168 177 Orgão emissor do documento de identificação - USANDO MTE*/
                                    'ORGAOEMIS' => array( 'tipo' => 'A', 'ini'  => 178, 'fim'  => 179 ),    /*178 179 Uf do orgão emissor do documento de identificação*/
                                    'DTIDENTIF' => array( 'tipo' => 'Y'/*'ini' => 180, 'fim' => 187 */),    /*180 187 Data da emissão do documento de identificação*/
                                    'DTNASCIME' => array( 'tipo' => 'Y'/*'ini' => 188, 'fim' => 195 */),    /*188 195 Data da emissão do documento de identificação*/
                                    'LOCALNASC' => array( 'tipo' => 'A', 'ini'  => 196, 'fim'  => 225 ),    /*196 225 Localidade do nascimento*/
                                    'UFLOCALNA' => array( 'tipo' => 'A', 'ini'  => 226, 'fim'  => 227 ),    /*226 227 UF da localidadade do nascimento*/
                                    'IDSEXOEMP' => array( 'tipo' => 'N', 'ini'  => 228, 'fim'  => 228 ),    /*228 228 Sexo 1 masculino e 2 feminino*/
                                    'NMMAEEMPR' => array( 'tipo' => 'A', 'ini'  => 229, 'fim'  => 268 ),    /*229 268 Nome da mãe*/
                                    'CDESTACIV' => array( 'tipo' => 'N', 'ini'  => 269, 'fim'  => 269 ),    /*269 269 Estado civil*/
                                    'CDREGICAS' => array( 'tipo' => 'N', 'ini'  => 270, 'fim'  => 270 ),    /*270 270 Regime de casamento*/
                                    'ENDERERES' => array( 'tipo' => 'A', 'ini'  => 271, 'fim'  => 300 ),    /*271 300 Endereço residêncial*/
                                    'NRENDERES' => array( 'tipo' => 'A', 'ini'  => 301, 'fim'  => 306 ),    /*301 306 Nro do endereço residêncial*/
                                    'COMPLERES' => array( 'tipo' => 'A', 'ini'  => 307, 'fim'  => 336 ),    /*307 336 complemento*/
                                    'BAIRRORES' => array( 'tipo' => 'A', 'ini'  => 337, 'fim'  => 356 ),    /*337 356 Bairro residêncial*/
                                    'DESCRIRES' => array( 'tipo' => 'A', 'ini'  => 357, 'fim'  => 386 ),    /*357 386 Descrição da localidade (município)*/
                                    'ESTADORES' => array( 'tipo' => 'A', 'ini'  => 387, 'fim'  => 388 ),    /*387 388 UF*/
                                    'NROCEPRES' => array( 'tipo' => 'N', 'ini'  => 389, 'fim'  => 396 ),    /*389 396 CEP*/
                                    'DDDLOCRES' => array( 'tipo' => 'N', 'ini'  => 397, 'fim'  => 400 ),    /*397 400 DDD da localidade residêncial*/
                                    'TELEFORES' => array( 'tipo' => 'N', 'ini'  => 401, 'fim'  => 408 ),    /*401 408 Telefone reidêncial*/
                                    'RESERVADO' => array( 'tipo' => 'A', 'ini'  => 409, 'fim'  => 410 ),    /*409 410 USO DO BANCO reservado para o processamento do arquivo*/
                                    'RESERERRO' => array( 'tipo' => 'A', 'ini'  => 411, 'fim'  => 470 ),    /*411 470 USO DO BANCO tabela de erros*/
                                    'RESERBRAN' => array( 'tipo' => 'A', 'ini'  => 471, 'fim'  => 506 ),    /*471 506 USO DO BANCO brancos*/
                                    'NRSEQUREG' => array( 'tipo' => 'N', 'ini'  => 507, 'fim'  => 512 ) );  /*507 512 Sequência do registro, iniciar em 000002*/
    private $layoutTrailler = array(
        'TIPOREGIS' => array( 'tipo' => 'N', 'ini' =>   1, 'fim' =>   1 ),  /*001 001 tipo de registro = 9*/
        'BRANCOS01' => array( 'tipo' => 'A', 'ini' =>   2, 'fim' =>   7 ),  /*002 007 brancos*/
        'VRSOMATOT' => array( 'tipo' => 'N', 'ini' =>   8, 'fim' =>  24 ),  /*008 024 soma dos valores*/
        'TABELAERR' => array( 'tipo' => 'A', 'ini' =>  25, 'fim' =>  84 ),  /*025 084 tabela de erros*/
        'BRANCOS02' => array( 'tipo' => 'A', 'ini' =>  85, 'fim' => 506 ),  /*085 506 brancos*/
        'NRSEQUREG' => array( 'tipo' => 'N', 'ini' => 507, 'fim' => 512 )   /*507 512 sequência do registro, último número*/
    );

    /**
     * Construtor. Adquiri os dados do formulário para as consultas
     * @param $params
     */
    public function __construct($entityManager, $hcmParameters, $connection,  $params){
        
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->nrForm       = $params['NRFORMCNAB'];    //Carrega os dados do formulário e FPA
        $this->nrOrg        = $params['NRORG'];
        $this->nrEvento     = $params['NREVENTOM'];
        $this->competencia  = FPA::getDtCompetencia();
        $this->tpMovimento  = $params['NRTIPOMOVIMENT'];
        $this->modalidCalc  = FPA::getNrModalidadeCalculo();
        $this->ocorrenCalc  = FPA::getOcorrenciaCalculo();
        $this->ordem        = $params['DSORDEM'];
        $this->dsIni        = $params['DSINICIO'];
        $this->dsFim        = $params['DSFIM'];
        $this->tpVinculo    = $params['DSTIPOVINCULO'];
        $this->tpInscr      = $params['IDTPINSCRESPONS'];
        $this->nrEstrutResp = $params['NRESTRUTRESPONS'];
        $this->idGeraMagnet = $params['IDGERAMEIOMAG'];
        $this->cdBanco      = $params['CDBANCO'];
        $this->idTeste      = $params['IDTESTE'];
        $this->cdConvenio   = $params['CODCONVENIO'];
        $this->cdOperador   = FPA::getCdOperador();

        $this->dtPagamento  = $params['DTPAGAMENTO'];
        $this->idUtilDtPgtoItem = $params['IDUTILDTPGTOITEM'];
        $this->idIgnoraOcorrencia = $params['IDIGNORAOCORRENCIA'];

        $this->idatualizacad = $params['IDATUALIZACAD'] == 'S' ? 7 : 1; //1 pagamento a empregados; 7 abertura de cadastro

        $this->nrSequencial = 1;    //Número sequencial
        $this->vrTotal      = 0;    //Valor total dos registros

        //Query do header
        $this->queryHeader = <<<SQL
SELECT  EML.NMESTRUTURAM   NMCONTRAT,
  EHL.CDCNPJESTRUT   CNPJCONTR,
  CCL.CDCONTCORR     CDCCCONTR,
  TO_CHAR(TO_DATE('$this->competencia'), 'MMYYYY') REFPAGAME
FROM ESTRUTURAM EML
JOIN ESTRUTURAH EHL ON EHL.NRORG        = EML.NRORG
                   AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                   AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE DTMESCOMPETENC <= '$this->competencia'
                                                AND NRORG           = EML.NRORG
                                                AND NRESTRUTURAM    = EML.NRESTRUTURAM )
LEFT JOIN CONTCORRPARC CCL ON CCL.NRORG          = EML.NRORG
                          AND CCL.NRPARCNEGOCIO  = EML.NRPARCNEGOCIO
                          AND CCL.CDBANCO        = '021'
                          AND CCL.NRCONTCORRPARC = ( SELECT MAX(NRCONTCORRPARC)
                                                     FROM CONTCORRPARC
                                                     WHERE NRORG         = CCL.NRORG
                                                       AND NRPARCNEGOCIO = CCL.NRPARCNEGOCIO
                                                       AND NRTPCONTCORR  = CCL.NRTPCONTCORR )
WHERE EML.NRORG        = $this->nrOrg
  AND EML.NRESTRUTURAM = $this->nrEstrutResp
SQL;

        //Query da remessa (body)
        $joinPartContcorparc = $this->idatualizacad == 7 ? 'LEFT' : ''; //Para pagamento a empregador (1) é obrighatório ter o banco, para abertura de cadastro (7) não é necessário ter conta.
        $queryBodyDtItem = $this->idUtilDtPgtoItem == 'N' ? ' ' : "AND ICF.DTOCORRENCIA = '".$this->dtPagamento."'"; //Filtra data de ocorrência do item de cálculo caso seja necessário
        
        if ($this->idIgnoraOcorrencia == 'S') {
            $queryBodyOcorrencia = '';
        } else {
            $queryBodyOcorrencia = "AND CF.NROCORRECAL    = $this->ocorrenCalc /*:P_OCORRECAL*/";
        }
        $this->queryBody = <<<SQL

SELECT
                                /*001 001 tipo de registro = 2*/
  VM.CDMATRICULA      CDMATRICU, /*002 010 matrícula do empregado*/
  /*ou usar este VM.NRVINCULOM CDMATRICU,*/ /*002 010 matrícula do empregado*/
  VM.NMVINCULOM       NMEMPREGA, /*011 110 Nome do empregado*/
                                /*111 111 forma de pagamento 3 ou 6 - USANDO 3*/
  REPLACE(CCV.CDAGENCIA,  '-')
                      CDAGENCIA, /*112 115 Código da agência da conta do beneficiário*/
  REPLACE(CCV.CDCONTCORR, '-')
                      CDCONTACO, /*116 126 Número da conta corrente*/
  PEH.NRCPFPESSOA     CDCPFEMPR, /*127 140 CPF do empregado*/
  ROUND(ICF.VREVENTOFOLHA, 2) * 100
                      VRPAGAMEN, /*141 151 Valor do pagamento*/
                                /*152 152 Tipo de documento de identificação 1 CI ou 5 CTPS/MT - USANDO 1*/
  /*LPAD(PEH.NRCTPSPESSOA, 9, '0') || '-' || LPAD(PEH.NRSERIECTPSPES, 5, '0')*/
  RPAD(PEH.NRRGPESSOA, 15, ' ')
                      IDENTIFIC, /*153 167 Identificação*/
                                /*168 177 Orgão emissor do documento de identificação - USANDO MTE*/
  PEH.SGUFCTPSPES     ORGAOEMIS, /*178 179 Uf do orgão emissor do documento de identificação*/
  PEH.DTCTPSPESSOA    DTIDENTIF, /*180 187 Data da emissão do documento de identificação*/
  PEH.DTNASCPESSOA    DTNASCIME, /*188 195 Data da emissão do documento de identificação*/
  MU.NMMUNICIPIO      LOCALNASC, /*196 225 Localidade do nascimento*/
  PEH.SGESTADO        UFLOCALNA, /*226 227 UF da localidadade do nascimento*/
  DECODE(PEH.IDSEXOPESSOA, 'M', '1', '2')
                      IDSEXOEMP, /*228 228 Sexo 1 masculino e 2 feminino*/
  PN.NMPRINCIPALPARC  NMMAEEMPR, /*229 268 Nome da mãe*/
  DECODE(PEH.CDESTACIVIL, 'S', '1', /*solteiro*/
                          'C', '2', /*casado*/
                          'D', '3', /*separado*/
                          'V', '4', /*viúvo*/
                          'I', '5', /*divorciado*/
                          'O', '6', /*outros*/
                          PEH.CDESTACIVIL) /*default (identificar erros)*/
                      CDESTACIV, /*269 269 Estado civil*/
  DECODE(PEH.CDESTACIVIL, 'C', '2', /*casado*/
                          '0')      /*default*/
                      CDREGICAS, /*270 270 Regime de casamento*/
  EP.DSENDERECO       ENDERERES, /*271 300 Endereço residêncial*/
  EP.NRIMOVELENDERECO NRENDERES, /*301 306 Nro do endereço residêncial*/
  EP.DSCOMPLEENDERECO COMPLERES, /*307 336 complemento*/
  EP.NMBAIRROENDERECO BAIRRORES, /*337 356 Bairro residêncial*/
  MU2.NMMUNICIPIO     DESCRIRES, /*357 386 Descrição da localidade (município)*/
  EP.SGESTADO         ESTADORES, /*387 388 UF*/
  REPLACE(REPLACE(EP.NRCEPENDERECO, '-'), '.')
                      NROCEPRES, /*389 396 CEP*/
  REPLACE(REPLACE(CP.CDPREFIXCOMUPARC, '('), ')')
                      DDDLOCRES, /*397 400 DDD da localidade residêncial*/
  REPLACE(REPLACE(CP.CDCOMUNICAPARC, '-'), ' ')
                     TELEFORES, /*401 408 Telefone reidêncial*/
                                /*409 410 USO DO BANCO reservado para o processamento do arquivo*/
                                /*411 470 USO DO BANCO tabela de erros*/
                                /*471 506 USO DO BANCO brancos*/
                                /*507 512 Sequência do registro, iniciar em 000002*/

  EHL.CDCNPJESTRUT   NRINSCCLI,
  CCL.CDCONTCORR     IDCLIENT,

  VM.NRVINCULOM      VINCULO,
  VM.NMVINCULOM      NOME,
  VH.NRESTRUTLEGAL   NROLEGAL,
  EHL.NMRAZSOCESTRUT RAZAOLEGAL
FROM GPE_VINCULOM VM
JOIN GPE_VINCULOH VH ON VH.NRORG      = VM.NRORG
                    AND VH.NRVINCULOM = VM.NRVINCULOM
                    AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                              WHERE DTMESCOMPETENC <= '$this->competencia' /*:P_COMPETENCIA*/
                                                AND NRORG           = VH.NRORG
                                                AND NRVINCULOM      = VH.NRVINCULOM )
/*Pessoa*/
JOIN GPE_PESSOA PE ON PE.NRORG    = VM.NRORG
                  AND PE.NRPESSOA = VM.NRPESSOA
JOIN GPE_PESSOAH PEH ON PEH.NRORG    = VM.NRORG
                    AND PEH.NRPESSOA = PE.NRPESSOA
                    AND PEH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                               WHERE DTMESCOMPETENC <= '$this->competencia' /*:P_COMPETENCIA*/
                                                 AND NRORG           = VM.NRORG
                                                 AND NRPESSOA        = PE.NRPESSOA )
/*conta corrente e banco do vínculo/pessoa*/
$joinPartContcorparc JOIN CONTCORRPARC CCV ON CCV.NRORG          = VM.NRORG
                     AND CCV.NRPARCNEGOCIO  = PE.NRPARCNEGOCIO
                     /*AND CCV.NRTPCONTCORR   = 2*/ /*1 Conta de Movimentação, 2 CNAB, 3 Conta Salário, 4 Pensão Judicial*/
                     AND CCV.IDATIVO        = 'S'
                     AND CCV.CDBANCO        = '021' --BANESTES
                     AND CCV.NRCONTCORRPARC = ( SELECT MAX(NRCONTCORRPARC)
                                                FROM CONTCORRPARC
                                                WHERE NRORG         = CCV.NRORG
                                                  AND IDATIVO       = 'S'
                                                  AND NRPARCNEGOCIO = CCV.NRPARCNEGOCIO
                                                  AND NRTPCONTCORR  = CCV.NRTPCONTCORR
                                                  AND CDBANCO       = CCV.CDBANCO )
/*município que o vínculo/pessoa nasceu*/
LEFT JOIN MUNICIPIO MU ON ( MU.NRORG = VM.NRORG OR MU.NRORG = 0 OR MU.NRORG IS NULL )
                      AND MU.CDPAIS      = PEH.CDPAIS
                      AND MU.SGESTADO    = PEH.SGESTADO
                      AND MU.CDMUNICIPIO = PEH.CDMUNICIPIO
/*Nome da mãe*/
LEFT JOIN RELACIONAPARC RP ON RP.NRORG           = VM.NRORG
                          AND RP.NRPARCNEGOCIO   = PE.NRPARCNEGOCIO
                          AND RP.NRTIPORELACIONA = 1 /*Mãe*/
LEFT JOIN PARCNEGOCIO PN ON PN.NRORG         = RP.NRORG
                        AND PN.NRPARCNEGOCIO = RP.NRPARCNEGRELAC
/*Endereço do vínculo/pessoa*/
LEFT JOIN ENDERECOPARC EP ON EP.NRORG          = VM.NRORG
                         AND EP.NRPARCNEGOCIO  = PE.NRPARCNEGOCIO /*alterar aqui*/
                         AND EP.CDTIPOENDERECO = 'PRINCIPAL'
LEFT JOIN MUNICIPIO MU2 ON ( MU2.NRORG = VM.NRORG OR MU2.NRORG = 0 OR MU2.NRORG IS NULL )
                        AND MU2.CDPAIS      = EP.CDPAIS
                        AND MU2.SGESTADO    = EP.SGESTADO
                        AND MU2.CDMUNICIPIO = EP.CDMUNICIPIO
/*forma de comunicação do vínculo/pessoa*/
LEFT JOIN COMUNICAPARC CP ON CP.NRORG         = VM.NRORG
                         AND CP.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                         AND CP.CDFORMACOMU   = '01' /*telefone principal*/
                         AND CP.NRCOMUNICAPARC = ( SELECT MAX(NRCOMUNICAPARC) FROM COMUNICAPARC /*garante que a forma de comunicação será única e a última cadastrada*/
                                                   WHERE NRORG         = VM.NRORG
                                                     AND NRPARCNEGOCIO = CP.NRPARCNEGOCIO
                                                     AND CDFORMACOMU   = CP.CDFORMACOMU)
/*Itens de cálculo*/
JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG          = VM.NRORG
                         AND ICF.NRVINCULOM     = VM.NRVINCULOM
                         AND ICF.VREVENTOFOLHA >= 0.005 /*Apenas eventos não zerados*/
                         AND ICF.NREVENTOM      = $this->nrEvento /*:P_EVENTO*/
                         $queryBodyDtItem
JOIN FPA_CALCULOFOLHA CF ON CF.NRORG          = ICF.NRORG
                        AND CF.NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
								        AND CF.NRTIPOMOVIMENT = $this->tpMovimento /*:P_TIPOMOVIMENT*/
								        AND CF.NRTPMODALIDCAL = $this->modalidCalc /*:P_TPMODALIDCAL*/
								        AND CF.DTOCORRENCIA   = '$this->competencia' /*:P_COMPETENCIA*/
								        $queryBodyOcorrencia
/*Estrutura Legal*/
LEFT JOIN ESTRUTURAM EML ON EML.NRORG        = VM.NRORG
                   AND EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
LEFT JOIN ESTRUTURAH EHL ON EHL.NRORG        = VM.NRORG
                   AND EHL.NRESTRUTURAM = VH.NRESTRUTLEGAL
                   AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE DTMESCOMPETENC <= '$this->competencia' /*:P_COMPETENCIA*/
                                                AND NRORG           = VM.NRORG
                                                AND NRESTRUTURAM    = EML.NRESTRUTURAM )
/*conta corrente e banco da estrutura legal*/
LEFT JOIN CONTCORRPARC CCL ON CCL.NRORG          = EML.NRORG
                          AND CCL.NRPARCNEGOCIO  = EML.NRPARCNEGOCIO
                          /*AND CCL.NRTPCONTCORR   = 2*/ /*1 Conta de Movimentação, 2 CNAB, 3 Conta Salário, 4 Pensão Judicial*/
                          AND CCL.CDBANCO        = '021'
                          AND CCL.IDATIVO        = 'S'
                          AND CCL.NRCONTCORRPARC = ( SELECT MAX(NRCONTCORRPARC)
                                                     FROM CONTCORRPARC
                                                     WHERE NRORG          = CCL.NRORG
                                                       AND IDATIVO        = 'S'
                                                       AND NRPARCNEGOCIO  = CCL.NRPARCNEGOCIO
                                                       AND NRCONTCORRPARC = CCL.NRCONTCORRPARC )
WHERE VM.NRORG = $this->nrOrg /*:P_NRORG*/
SQL;

        $where = '';     //Query da remessa - Ajusta o complemento do where para filtrar e ordenar
        if($this->tpVinculo != ''){ //Filtra os tipos de vínculos (se necessáio)
            $this->tpVinculo = ','.$this->tpVinculo.','; //Ajsute para a query funcionar
            $where = "\r\n  AND INSTR('$this->tpVinculo', ','||VM.NRTIPOVINCULOM||',' ) > 0";
        }
        switch($this->ordem){
            case 'ALFABETICA':
                $where .= "\r\n  AND VM.NMVINCULOM BETWEEN '$this->dsIni' AND '$this->dsFim'\r\nORDER BY VM.NMVINCULOM";
                break;
            case 'MATRICULA':
                $where .= "\r\n  AND VM.CDMATRICULA BETWEEN '$this->dsIni' AND '$this->dsFim'\r\nORDER BY VM.CDMATRICULA";
                break;
            case 'DATA_ADMISSAO':
                $where .= "\r\n  AND VM.DTADMISSAOVINC BETWEEN '$this->dsIni' AND '$this->dsFim'\r\nORDER BY VM.DTADMISSAOVINC";
                break;
            case 'ESTRUTURA':
                $where .= "\r\n  AND VH.NRESTRUTLEGAL = '$this->dsIni'";
                break;
            case 'LISTA_ESTRUTURA':
                if(isset($this->dsIni))
                    $where .= "\r\n  AND VH.NRESTRUTLEGAL = '$this->dsIni'";
                break;    
            case 'LISTA_VINCULO':
                if ($this->dsIni == 'A') {
                    $where .= "\r\n  AND '" . $this->dsIni . "' = 'A'";
                } elseif ($this->dsIni == 'N') {
                    $where .= "\r\n  AND VM.NRVINCULOM = '" . $this->dsIni . "'";
                } else {
                    if(is_array($this->dsIni) && !empty($this->dsIni)){
                        $dsIni = implode(',',$this->dsIni);
                        $where .= "\r\n  AND VM.NRVINCULOM IN ($dsIni)";
                    } else {
                        $where .= "\r\n  AND VM.NRVINCULOM IN ($this->dsIni)";
                    }
                }
                break;
            case 'ESTRUTURA_SIND':
                if ($this->dsIni == 'A') {
                    $where .= "\r\n  AND '" . $this->dsIni . "' = 'A'";
                } elseif ($this->dsIni == 'N') {
                    $where .= "\r\n  AND VH.NRESTRUTSIND = '" . $this->dsIni . "'";
                } else {
                    $where .= "\r\n  AND VH.NRESTRUTSIND IN (" . $this->dsIni . ")";
                }
                break;
            case 'LISTA_NRESTRUTSIND':
                if($this->dsIni)
                $where .= "\r\n  AND VH.NRESTRUTLEGAL = '$this->dsIni'";
                if(isset($this->dsFim))
                $this->dsFim = is_array($this->dsFim) ? implode($this->dsFim, ",") : $this->dsFim;
                $where .= "\r\n  AND VH.NRESTRUTSIND IN (".$this->dsFim. ")";
                break;
            default:
                throw new \Exception("Opção de ordenamento inválida ($this->ordem).");
        }
        $this->queryBody .= $where; //Completando a query com o restante do where e do order by
        $this->queryBody = "
            SELECT
                CDMATRICU, NMEMPREGA, CDAGENCIA, CDCONTACO, CDCPFEMPR, SUM(VRPAGAMEN) VRPAGAMEN,
                IDENTIFIC, ORGAOEMIS, DTIDENTIF, DTNASCIME, LOCALNASC UFLOCALNA, IDSEXOEMP,
                NMMAEEMPR, CDESTACIV, CDREGICAS, ENDERERES, NRENDERES, BAIRRORES, DESCRIRES,
                ESTADORES, NROCEPRES, DDDLOCRES, TELEFORES, NRINSCCLI, IDCLIENT, VINCULO,
                NOME, NROLEGAL, RAZAOLEGAL
            FROM ($this->queryBody)
            GROUP BY
                CDMATRICU, NMEMPREGA, CDAGENCIA, CDCONTACO, CDCPFEMPR, IDENTIFIC, ORGAOEMIS,
                DTIDENTIF, DTNASCIME, LOCALNASC, UFLOCALNA, IDSEXOEMP, NMMAEEMPR, CDESTACIV,
                CDREGICAS, ENDERERES, NRENDERES, BAIRRORES, DESCRIRES, ESTADORES, NROCEPRES,
                DDDLOCRES, TELEFORES, NRINSCCLI, IDCLIENT, VINCULO, NOME, NROLEGAL, RAZAOLEGAL";
    }

    /**
     * Verifica se os campos obrigatórios estão preenchidos, caso contrário lança um erro
     * @throws Exception
     */
    private function verificaCamposRequeridos(){
        if($this->ordem == 'ALFABETICA' || $this->ordem == 'MATRICULA')
            $camposReq = array( 'Nº do evento'                      => $this->nrEvento,
                                'Tipo de movimento'                 => $this->tpMovimento,
                                'Ordem'                             => $this->ordem,
                                'Descrição inicial'                 => $this->dsIni,
                                'Descrição final'                   => $this->dsFim/*,
                                'Estrutura da Pessoa Responsável'   => $this->nrEstrutResp*/ );
        else
            $camposReq = array( 'Nº do evento'                      => $this->nrEvento,
                                'Tipo de movimento'                 => $this->tpMovimento,
                                'Ordem'                             => $this->ordem,
                                'Descrição inicial'                 => $this->dsIni);
        $camposVazio = '';
        foreach($camposReq as $label => $campo ) //Procura campos requeridos não preenchidos
            if($campo == null)
                $camposVazio .= $label."\r\n";
        if($camposVazio != '')
            throw new \Exception("Os seguintes campos devem estar preenchidos:\r\n".$camposVazio);
    }

    /**
     * Verifica se os dados obrigatórios de resposta da query estão todos preenchidos
     * @param array $resultQuery
     * @throws Exception
     */
    private function verificaResultQuery(array $resultQuery){
        $required = array( 'CDMATRICU' => 'Matrícula do empregado',
            'NMEMPREGA' => 'Nome do empregado',
            'CDAGENCIA' => 'Código da agência da conta do beneficiário',
            'CDCONTACO' => 'Número da conta corrente',
            'CDCPFEMPR' => 'CPF do empregado',
            'VRPAGAMEN' => 'Valor do pagamento',
            'IDENTIFIC' => 'Identificação',
            'ORGAOEMIS' => 'Uf do orgão emissor do documento de identificação*',
            'DTIDENTIF' => 'Data da emissão do documento de identificação',
            'DTNASCIME' => 'Data da emissão do documento de identificação',
            'LOCALNASC' => 'Localidade do nascimento',
            'UFLOCALNA' => 'UF da localidadade do nascimento',
            'IDSEXOEMP' => 'Sexo 1 masculino e 2 feminino',
            'NMMAEEMPR' => 'Nome da mãe',
            'CDESTACIV' => 'Estado civil',
            'CDREGICAS' => 'Regime de casamento',
            'ENDERERES' => 'Endereço residencial',
            'NRENDERES' => 'Nro do endereço residencial',
            'BAIRRORES' => 'Bairro residencial',
            'DESCRIRES' => 'Descrição da localidade (município)',
            'ESTADORES' => 'UF',
            'NROCEPRES' => 'CEP'/*,
            'DDDLOCRES' => 'DDD da localidade residencial',
            'TELEFORES' => 'Telefone residencial'*/ );
        $missing = '';
        $nrvinc = -1;
        $vn = false;            //Flag da quebra de linha do vínculo
        $vr = false;            //Contador/Flag do uso da vírgula
        foreach($resultQuery as $regKey => $regVal){    //Percorre todos os resultados da query
            foreach($regVal as $resKey => $resVal){     //Percorre todos os campos de um registro da query
                switch($resKey){                        //Verifica se é referente à atualização, neste caso o mesmo...
                    case 'CDAGENCIA':                   //...irá ignorar a agência e a conta corrente
                    case 'CDCONTACO':
                        if($this->idatualizacad == '7')
                            $resVal .= '';
                }
                if(!isset($resVal) && isset($required[$resKey])){       //Procura resultados vazios e se encontar verifica se o mesmo é requerido
                    if($nrvinc != $resultQuery[$regKey]['VINCULO']){    //Verifica se os erros 'trocaram de vínculo'
                        $nrvinc   = $resultQuery[$regKey]['VINCULO'];   //Impede que a frase acima apareça repetidas vezes para o mesmo vínculo
                        $nmvinc   = $resultQuery[$regKey]['NOME'];      //Nome do vínculo
                        $vr       = false;                              //Reinicia o uso da vírgula
                        if($vn){
                            $missing .= self::newLine."Vínculo $nrvinc $nmvinc: ";  //Identifica o vínculo em que ocorreu o(s) erro(s)
                        } else {
                            $missing .= "Vínculo $nrvinc  $nmvinc: ";   //Primeira lina que identifica o vínculo em que ocorreu o(s) erro(s)
                            $vn = true;                                 //Sinaliza que as demais linhas de identificação do vínculo são a 2ª linha em diante
                        }
                    }
                    if($vr){                                            //Verirfica se é o caso de se utilizar a ', '
                        $missing .= ', ';
                    } else {
                        $vr = true;                                     //Sinaliza o uso da vírgula para o próximo caso (vínculo)
                    }
                    $missing .= $required[$resKey];                     //Atualiza a mensagem de erro/campo faltante
                }
            }
        }
        if($missing != '')
            throw new \Exception("Os seguintes campos devem estar preenchidos:".self::newLine.$missing);
    }
    /**
     * Adquiri os dados da remessa (bodyResult)
     * @return array
     * @throws Exception
     */
    private function getHeaderQueryResult(){
        try{
            return $this->connection->fetchAll($this->queryHeader);
            //return $this->entityManager->fetchAll($this->queryHeader);
        } catch(\Exception $e) {
            throw new \Exception('Erro na query, parte da remessa, a query usada foi esta: ' . $this->queryHeader);
        }
    }

    /**
     * Adquiri os dados fixos, do formulário e computados do cabeçalho (header)
     * @return array
     */
    private function getHeaderFixed(){
        $dtGravacao = DateUtil::getDataAtual();
        $dtGravacao = \HCM\Util\DateUtil::getDataDeString($dtGravacao);

        $dtPagament = substr($this->dtPagamento, 0, 10); //Somente a data, sem o horário
        $dtPagament = \HCM\Util\DateUtil::getDataDeString($dtPagament);

        //$hrGravacao = str_replace(':', '', substr(now(), 11, 8));

        return array(   'TIPOREGIS' => 1,                       //001 001 Tipo de registro = 1
                        'CDCONTRAT' => $this->cdConvenio,       //002 006 Código do contratante --ANTES EU USAVA O CDINTESTRUTURA da estrutura, via query
                        /*NMCONTRAT*/                           //007 036 Nome do contratante
                        /*CNPJCONTR*/                           //037 050 CNPJ do contratante
                        /*CDCCCONTR*/                           //051 061 Conta corrente do contratante
                        'DTGRAVACA' => $dtGravacao,             //062 069 Data de Gravação do Arquivo AAAAMMDD
                        'TIPOREMES' => $this->idatualizacad,    //070 070 Tipo de remessa 1 pagamento 7 cadastro
                        'ORIGEMARQ' => 'FP3',                   //071 073 Origem do arquivo = FP3
                        /*REFPAGAME*/                           //074 079 Referencia do pagamento
                        'DTPAGAMEN' => $dtPagament,             //080 087 Data de crédito
                        'TABELAERR' => ' ',                     //088 147 uso do banco, numérico,  tabela de erros
                        'BRANCOS01' => ' ' );                   //148 506 brancos
                        /*'NRSEQUREG' => null );*/                  //507 512 Sequência do registro = 000001
    }
    /**
     * Adquiri os dados da remessa (bodyResult)
     * @return array
     * @throws Exception
     */
    private function getBodyQueryResult(){
        try{
            return $this->connection->fetchAll($this->queryBody);
        } catch(\Exception $e) {
            throw new \Exception('Erro na query, parte da remessa, a query usada foi esta: ' . $this->queryBody);
        }
    }

    /**
     * Adquiri os dados fixos, do formulário e computados da remessa (body)
     * @return array
     * @throws Exception
     */
    private function getBodyFixed(){
        $dtPagamento = substr($this->dtPagamento, 0, 10);
        $dtPagamento = \HCM\Util\DateUtil::getDataDeString($dtPagamento);
        return array(   'TIPOREGIS' => 2,                       //001 001 tipo de registro = 2
                        /*CDMATRICU*/                           //002 010 matrícula do empregado
                        /*NMEMPREGA*/                           //011 110 Nome do empregado
                        'CDFORMPAG' => 3,                       //111 111 forma de pagamento 3 ou 6 - USANDO 3
                        /*CDAGENCIA*/                           //112 115 Código da agência da conta do beneficiário
                        /*CDCONTACO*/                           //116 126 Número da conta corrente
                        /*CDCPFEMPR*/                           //127 140 CPF do empregado
                        /*VRPAGAMEN*/                           //141 151 Valor do pagamento
                        'TPDOCUMEN' => 1,                       //152 152 Tipo de documento de identificação 1 CI ou 5 CTPS/MT - USANDO 1
                        /*IDENTIFIC*/                           //153 167 Identificação
                        'ORGEMISID' => 'SSP',                   //168 177 Orgão emissor do documento de identificação - USANDO MTE
                        /*ORGAOEMIS*/                           //178 179 Uf do orgão emissor do documento de identificação
                        /*DTIDENTIF*/                           //180 187 Data da emissão do documento de identificação
                        /*DTNASCIME*/                           //188 195 Data da emissão do documento de identificação
                        /*LOCALNASC*/                           //196 225 Localidade do nascimento
                        /*UFLOCALNA*/                           //226 227 UF da localidadade do nascimento
                        /*IDSEXOEMP*/                           //228 228 Sexo 1 masculino e 2 feminino
                        /*NMMAEEMPR*/                           //229 268 Nome da mãe
                        /*CDESTACIV*/                           //269 269 Estado civil
                        /*CDREGICAS*/                           //270 270 Regime de casamento
                        /*ENDERERES*/                           //271 300 Endereço residêncial
                        /*NRENDERES*/                           //301 306 Nro do endereço residêncial
                        /*COMPLERES*/                           //307 336 complemento
                        /*BAIRRORES*/                           //337 356 Bairro residêncial
                        /*DESCRIRES*/                           //357 386 Descrição da localidade (município)
                        /*ESTADORES*/                           //387 388 UF
                        /*NROCEPRES*/                           //389 396 CEP
                        /*DDDLOCRES*/                           //397 400 DDD da localidade residêncial
                        /*TELEFORES*/                           //401 408 Telefone reidêncial
                        'RESERVADO' => ' ',                     //409 410 USO DO BANCO reservado para o processamento do arquivo
                        'RESERERRO' => ' ',                     //411 470 USO DO BANCO tabela de erros
                        'RESERBRAN' => ' '                      //471 506 USO DO BANCO brancos
                        /*NRSEQUREG*/);                         //507 512 Sequência do registro, iniciar em 000002

    }

    /**
     * @return array
     * Array da parte 'fixa' do layout
     */
    private function getTraillerFixed(){
        return array(   'TIPOREGIS' => 9,               /*001 001 tipo de registro = 9*/
                        'BRANCOS01' => ' ',             /*002 007 brancos*/
                        'VRSOMATOT' => $this->vrTotal,  /*008 024 soma dos valores*/
                        'TABELAERR' => ' ',             /*025 084 tabela de erros*/
                        'BRANCOS02' => ' ',             /*085 506 brancos*/
                        /*NRSEQUREG*/ );                /*507 512 sequência do registro, último número*/
    }

    /**
     * Mescla os dados fixos com o resultado da query em um novo array
     * @param array $dadosFixo
     * @param array $dadosResult
     * @return array
     */
    private function mesclaDadosFixoResult($dadosFixo, $dadosResult){
        foreach($dadosResult as $linha => $lixo) {
            foreach ($dadosFixo as $linha2 => $registros) {
                $dadosResult[$linha][$linha2] = $registros;
            }
            $dadosResult[$linha]['NRSEQUREG'] = $this->nrSequencial++; //Número sequencial do registro, todos os arrays (header, body e trailler) devem ter este campo: NRSEQUREG
        }
        return $dadosResult;
    }

    /**
     * @throws Exception
     * Adquiri os dados para geração do magnético e do arquivo
     */
    public function gerar(){
        $this->verificaCamposRequeridos();              //Se houver algum campo requerido não preenchido este emitirá uma exceção
        $this->verificaMagnetico();                     //Retorna uma exceção caso já existam registros no magnético

        $Result = $this->getHeaderQueryResult();    //Adquiri o resultado via query do header
        $Fixo   = $this->getHeaderFixed();          //Adquiri os campos fixos do header
        $this->headerResult = $this->mesclaDadosFixoResult($Fixo, $Result);

        $Result = $this->getBodyQueryResult();      //Adquiri o resultado via query do body (remessa)
        
        $this->verificaResultQuery($Result);        //Verifica/gerar execeção com relatórios de erros

        $Fixo   = $this->getBodyFixed();            //Adquiri os campos fixos do body
        $this->bodyResult = $this->mesclaDadosFixoResult($Fixo, $Result);

        //Obtendo a string de saída de acordo com o layout - HEADER
        $campo = $this->headerResult[0];
        $this->headerString = Util::formataLayoutString($campo, $this->layoutHeader);

        //Obtendo a string de saÍda de acordo com o layout - BODY
        foreach( $this->bodyResult as $campo) {
            $this->vrTotal += (int)$campo['VRPAGAMEN'];
            $this->bodyString[] = Util::formataLayoutString($campo, $this->layoutBody);
        }

        //Obtendo a string de saída de acordo com o layout - TRAILLER
        $Fixo = $this->getTraillerFixed();
        $this->traillerResult = $this->mesclaDadosFixoResult($Fixo, array(null));
        $campo = $this->traillerResult[0];
        $this->traillerString = Util::formataLayoutString($campo, $this->layoutTrailler);

        //Persiste no magnético (se for o caso)
        $this->gerarMagnetico();
    }

    /**
     * @param string $nomeArquivo
     * @param string $extensao
     * @param string $caminho
     * @throws Exception
     * Gera o arquivo cnab
     */
    public function gerarArquivo($nomeArquivo, $extensao){
        
        //Se existirem dados da organização e operador este usará no nome do arquivo
        if($this->nrOrg)  $nomeArquivo .= '-'.(string)$this->nrOrg;
        if(FPA::getCdOperador()) $nomeArquivo .= '-'.(string)FPA::getCdOperador();
        $nomeArquivo .= '.'.(string)$extensao;
        FPA::escreveArquivo($nomeArquivo, $this->headerString, true);
        FPA::escreveArquivo($nomeArquivo, $this->bodyString, true);
        FPA::escreveArquivo($nomeArquivo, $this->traillerString, true);
        
        return array(
            'ARQUIVO' => $nomeArquivo,
            'CALCULO' => null,
            'MENSAGENS' => null,
            'ERRO' => null,
            'ABORT' => null
        );
        
    }

    /**
     * @throws Exception
     * Verifica se já existem registros no magnético (somente se a gravação do mesmo foi requerida). A verificação é
     * antecipada para evitar processamento (dos results e arrays de string) desnecessário.
     */
    public function verificaMagnetico(){
        if($this->idGeraMagnet == 'S' && $this->idTeste == 'N') { //Verifica o magnético somente se a gravação do mesmo foi requerida
            $magnetico = new FpaCnabmagnetEntity($this->entityManager, $this->hcmParameters);
            $aux = $magnetico->retornaRegistros($this->competencia, $this->nrOrg, $this->cdBanco, $this->tpMovimento, $this->cdOperador);
            if ($aux)
                throw new \Exception("Já existem registros no magnético. O CNAB não foi gerado.");
        }
    }
    /**
     * @throws Exception
     * @throws \TkValidationException
     * Grava dados no magnético somente se estes foram requeridos
     */
    public function gerarMagnetico(){
        if($this->idGeraMagnet == 'S' && $this->idTeste == 'N'){ //Grava no magnético somente se o mesmo foi requerido
            $magnetico = new FpaCnabmagnetEntity($this->entityManager, $this->hcmParameters);
            for($i = 0, $n = count($this->bodyResult); $i < $n; ++$i) {
                $magnetico->insereRegistroLista($this->nrOrg, $this->cdOperador, $this->competencia, null, $this->tpInscr,
                    $this->bodyResult[$i]['CDCPFEMPR'], $this->bodyString[$i], '021', $this->tpMovimento,
                    $this->bodyResult[$i]['NRSEQUREG'], $this->bodyResult[$i]['VRPAGAMEN'] / 100,
                    $this->bodyResult[$i]['VINCULO']);
                $magnetico->persist();
            }
        }
    }

    /**
     * Retrocede os registros do magnético
     */
    public function retroceder(){
        $magnetico = new FpaCnabmagnetEntity($this->entityManager, $this->hcmParameters);
        $magnetico->retroceder($this->nrOrg, $this->competencia, $this->cdBanco, $this->tpMovimento, $this->cdOperador);
    }

    /**
     * Retorna um array de array contendo todos os campos de resultado da query
     * @return mixed
     */
    public function getHeaderResult(){
        return $this->headerResult;
    }

    /**
     * Retorna um array de array contendo todos os campos de resultado da query
     * @return mixed
     */
    public function getBodyResult(){
        return $this->bodyResult;
    }

    /**
     * Retorna um array de array contendo todos os campos de resultado da query
     * @return mixed
     */
    public function getTraillerResult(){
        return $this->traillerResult;
    }

    /**
     * Retorna um array contendo a string do header (cabeçalho) formatada para geração do arquivo
     * @return array
     */
    public function getHeaderString(){
        return $this->headerString;
    }

    /**
     * Retorna um array contendo as strings do body (dados de tranferências) formatada para geração do arquivo
     * @return array
     */
    public function getBodyString(){
        return $this->bodyString;
    }

    /**
     * Retorna um array contendo a string do trailler (final do arquivo) formatada para geração do arquivo
     * @return array
     */
    public function getTraillerString(){
        return $this->traillerString;
    }
} 