<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;
use HCM\Util\DateUtil;
use HCM\FPA\Repository\GozoFeriasRepository;
use HCM\Geral\Repository\AlteSituacaoFuncionalRepository;
use HCM\Geral\Services\ESocialService;
use Teknisa\Model\Gestaopessoas\Entities\GpeSitufuncm;

class GozoFeriasService {
    
    protected $entityManager;
    protected $hcmParameters;
    protected $gozoFeriasRepository;
    protected $alteSituacaoFuncionalRepository;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, GozoFeriasRepository $gozoFeriasRepository, AlteSituacaoFuncionalRepository $alteSituacaoFuncionalRepository, ESocialService $eSocialService) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->gozoFeriasRepository = $gozoFeriasRepository;
        $this->eSocialService = $eSocialService;
        $this->alteSituacaoFuncionalRepository = $alteSituacaoFuncionalRepository;
    }
    
    public function deleteGozoFerias($dataset) {
        $nrFerias = $dataset["NRFERIAS"];
        $gozoFerias = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->findBy(array("nrferias" => $nrFerias, "nrorg" => $dataset['NRORG']));
        if (isset($gozoFerias[0])) {
            throw new \Exception(Messages::HCM01000_FPA_FERIAS_EXISTS_GOZOFERIAS_ERROR, 10);
        }
        else {
            $this->entityManager->beginTransaction();
            $ferias = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->findOneBy(array("nrferias" => $nrFerias, "nrorg" => $dataset['NRORG']));
            $this->entityManager->remove($ferias);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    public function verificaPossibilidadeAlteracao($nrchave, $edicaoexclusao) {
        $edicaodisponivel = $this->eSocialService->verificaPossibilidadeAlteracao($nrchave, 'Gozo de Ferias', $edicaoexclusao);
         
        if ($edicaodisponivel === true){
            return true;
        } else {
            $message = Messages::evaluateMessage(MESSAGES::HCM00000_EDITAR_EXCLUIR_REGISTRO_AFASTAMENTO, $nrchave);
            return new \Zeedhi\Framework\DTO\Response\Message($message, DTO\Response\Message::TYPE_WARNING);
        }
    }
    
    public function validaGozoFerias($dataset) {
        
        $nrVinculom = $dataset["NRVINCULOM"];
        $nrgozoferias = $dataset['NRGOZOFERIAS'];
        $inigozoatual = DateUtil::getDataDeString($dataset['DTINIGOZOFERIAS']); 
        $fimgozoatual = DateUtil::getDataDeString($dataset['DTFIMGOZOFERIAS']);
        $nrorg = $dataset['NRORG'];
        $gozoFerias = $this->gozoFeriasRepository->retornaGozoFerias($nrorg,$nrVinculom);
        foreach($gozoFerias as $periodo){
            $inigozo =  DateUtil::getDataDeString($periodo['DTINIGOZOFERIAS'],DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            $fimgozo =  DateUtil::getDataDeString($periodo['DTFIMGOZOFERIAS'],DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            if ($nrgozoferias == $periodo['NRGOZOFERIAS']) {
                continue;
            }
            $datas = array(DateUtil::truncateString($periodo['DTINIGOZOFERIAS']),DateUtil::truncateString($periodo['DTFIMGOZOFERIAS']));
            if ($inigozoatual) {
                if (DateUtil::comparaDatas($inigozoatual,$inigozo,DateUtil::DEPOIS) && DateUtil::comparaDatas($inigozoatual,$fimgozo,DateUtil::ANTES) || 
                    DateUtil::comparaDatas($fimgozoatual,$inigozo,DateUtil::DEPOIS) && DateUtil::comparaDatas($fimgozoatual,$fimgozo,DateUtil::ANTES)) {
                    throw new \Exception(Messages::evaluateMessage(Messages::HCM01000_FPA_GOZOFERIAS_VINCPOSSUIPERIODO_ERROR,$datas),10);
                }
            }
        }
    }
    
    public function avisaGozoAfastados($dataset) {
        
        $nrVinculom = $dataset["NRVINCULOM"];
        $nrgozoferias = $dataset['NRGOZOFERIAS'];
        $inigozoatual = DateUtil::getDataDeString($dataset['DTINIGOZOFERIAS']); 
        $fimgozoatual = DateUtil::getDataDeString($dataset['DTFIMGOZOFERIAS']);
        $nrorg = $dataset['NRORG'];
        $alteSituFunc = $this->alteSituacaoFuncionalRepository->retornaUltimaSituacaoPorDatainicio($nrorg, $nrVinculom, $fimgozoatual);
        if ($alteSituFunc) {
            // Verifica se não são as situações Normal e Atestado médico
            if (($alteSituFunc->getNrsitufuncm() != GpeSitufuncm::SITUACAO_FUNCIONAL_NORMAL) &&
                ($alteSituFunc->getNrsitufuncm() != GpeSitufuncm::SITUACAO_FUNCIONAL_ATESTADO)) {
                    
                // Verifica se o período de gozo e o período do afastamento se intercalam
                $dtinisitufunc = $alteSituFunc->getDtinisitufunc();
                if ($alteSituFunc->getDtfimsitufunc())
                    $dtfimsitufunc = $alteSituFunc->getDtfimsitufunc();
                else
                    $dtfimsitufunc = $fimgozoatual; 
                if (DateUtil::comparaDatas($inigozoatual,$dtinisitufunc,DateUtil::DEPOIS) && DateUtil::comparaDatas($inigozoatual,$dtfimsitufunc,DateUtil::ANTES) || 
                    DateUtil::comparaDatas($fimgozoatual,$dtinisitufunc,DateUtil::DEPOIS) && DateUtil::comparaDatas($fimgozoatual,$dtfimsitufunc,DateUtil::ANTES)){
                        $dataAlteSituFunc = array(DateUtil::truncateString($dtinisitufunc),DateUtil::truncateString($dtfimsitufunc));
                        return $dataAlteSituFunc;
                }
            }
        }
        return false;
    }
}