<?php

namespace HCM\GSO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\GSO\Logic\PontuacaoOcupacao;

class OcupacaoGestaoController extends Crud {

    protected $dataSourceName = 'gso_pontuacaocupa';
    protected $dataSourceNamePontuaGet = 'gso_getPontuacaocupa';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NROCUPACAOM', 'P_NMOCUPACAOH', 'P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    /*Pontuação da ocupação*/
    
    public function getPontuaOcupacao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        foreach($filterData->getConditions() as $conditions){
            if($conditions['columnName'] === 'P_NROCUPACAOM')
                $nrocupa = $conditions['value']; 
            if($conditions['columnName'] === 'P_NMOCUPACAOH')
                $nmocupa = $conditions['value'];
        }
        $filterData->setDataSourceName($this->dataSourceNamePontuaGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $pontuacaoOcupacaoLogic = new PontuacaoOcupacao($this->hcmParameters);
        $dados = $pontuacaoOcupacaoLogic->PreparaPontuaOcupacao($dataSet, $nrocupa, $nmocupa);
        $response->addDataSet(new DataSet($dataSet->getDataSourceName(), $dados));
    }
    
    public function setPontuaOcupacao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRPONTUACAOCUPA', $row)) {
                    if ($row['NRPONTUACAOCUPA'] === '') {
                        $row['NRPONTUACAOCUPA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GSO_PONTUACAOCUPA', 1, 12, true); 
                    }
                } else {
                    $row['NRPONTUACAOCUPA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GSO_PONTUACAOCUPA', 1, 12, true); 
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deletePontuaOcupacao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    /*Formação da ocupação*/
    
    public function getFormacaoOcupa(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = "gso_getFormacaocupa";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setFormacaoOcupa(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "gso_formacaocupa";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFORMACAOCUPA', $row)) {
                    if ($row['NRFORMACAOCUPA'] === '') {
                        $row['NRFORMACAOCUPA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GSO_FORMACAOCUPA', 1, 12, true); 
                    }
                } else {
                    $row['NRFORMACAOCUPA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GSO_FORMACAOCUPA', 1, 12, true); 
                }
                $row['NRAREAFORMACAO'] = isset($row['NRAREAFORMACAO'])?$row['NRAREAFORMACAO']:null;
                $row['CDIDIOMA'] = isset($row['CDIDIOM'])?$row['CDIDIOM']:null;
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteFormacaoOcupa(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = "gso_formacaocupa";
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    /*Perfil Profissiográfico da ocupção*/
    
    public function getRelacocupapppGso(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = "sst_getRelacocupappp";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    
    public function setRelacocupapppGso(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_relacocupappp";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRRELACOCUPAPPP', $row)) {
                    if ($row['NRRELACOCUPAPPP'] === '') {
                        $row['NRRELACOCUPAPPP'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_RELACOCUPAPPP', 1, 12, true); 
                    }
                } else {
                    $row['NRRELACOCUPAPPP'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_RELACOCUPAPPP', 1, 12, true); 
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteRelacocupapppGso(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = "sst_relacocupappp";
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    /* Avaliação da Ocupação*/
    
    public function getAvaliaOcupa(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = "gso_getAvalocupacao";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    
    public function setAvaliaOcupa(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "gso_avalocupacao";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRAVALOCUPACAO', $row)) {
                    if ($row['NRAVALOCUPACAO'] === '') {
                        $row['NRAVALOCUPACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GSO_AVALOCUPACAO', 1, 12, true); 
                    }
                } else {
                    $row['NRAVALOCUPACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GSO_AVALOCUPACAO', 1, 12, true); 
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteAvaliaOcupa(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = "gso_avalocupacao";
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 