<?php

namespace HCM\GSO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\GSO\Logic\TabelaSalarialLogic;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\HTTP\Kernel;
use HCM\Util\Messages;

class TabelaSalarialController extends Crud {

    protected $dataSourceName = 'gso_tabelasalarial';
    protected $dataSourceNameGet = 'gso_getTabelasalarial';
    protected $dataSourceNameGetCustom = 'getCustomocupacao';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_EXISTEVINC', 'P_NRESTRUTLEGAL', 'P_NRESTRUTGEREN', 'P_NRESTRUTSIND');
    public function __construct ( Manager $dataSourceManager, Kernel $httpKernel, EntityManager $entityManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->httpKernel    = $httpKernel;
        $this->entityManager = $entityManager;
        $this->parameterBag  = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getFaixaSalario(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setFaixaSalario(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRTABELASALARIAL', $row)) {
                    if ($row['NRTABELASALARIAL'] === '') {
                        $row['NRTABELASALARIAL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GSO_TABELASALARIAL', 1, 12, true); 
                    }
                } else {
                    $row['NRTABELASALARIAL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GSO_TABELASALARIAL', 1, 12, true); 
                }
                $row['NRESTRUTSIND'] = isset($row['NRESTRUTSIND']) ? $row['NRESTRUTSIND'] : NULL;
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteFaixaSalario(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function exportarTemplateSal(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataset = $request->getDataSet()->getRows();
            $row = $dataset[0];
            $TabelaSalarialLogic = new TabelaSalarialLogic($this->httpKernel, $this->entityManager, $this->hcmParameters);
            $file = $TabelaSalarialLogic->exportarTemplateSal($row);
            if($file){
                $return = array('NMARQUIVO' => $file);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $return));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importarTemplateSal(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataset = $request->getDataSet()->getRows();
            $row = $dataset[0];
            $TabelaSalarialLogic = new TabelaSalarialLogic($this->httpKernel, $this->entityManager, $this->hcmParameters);
            $messages = $TabelaSalarialLogic->importarTemplateSal($row);
            
            $response->addNotification(new DTO\Response\Notification(Messages::HCM00001_GER_IMPORTACAO_SUCESS));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getCustomocupacao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGetCustom);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 