<?php

namespace HCM\Geral\Logic;

use HCM\Geral\Logic\Esocial\ESocialDataFactory;
use HCM\Geral\Logic\Esocial\ESocialTransmissor;
use HCM\Geral\Logic\Esocial\ESocialXmlFactory;
use HCM\Util\DateUtil;
use HCM\Util\FileUploader;
use HCM\Util\Messages;
use HCM\Util\Repositories;

/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 11/10/2017
 * Time: 10:00
 */

class ESocialLogic {

    /* Grupos de Eventos do eSocial */
    const IDGRUPO_INICIAL = 'I';
    const IDGRUPO_TABELAS = 'T';
    const IDGRUPO_NAOPERIODICO = 'N';
    const IDGRUPO_PERIODICO = 'P';

    /* Operações Tela Envios */
    const IDTRANSENVIO = 'ENVIAR'; //Enviar Evento
    const IDTRANSCONSU = 'CONSULTAR'; //Consultar Status do evento
    const IDTRANSREENV = 'REENVIAR'; //Reenviar Evento
    const IDTRANSEXCLU = 'EXCLUSAO'; //Excluir Evento
    /* Operações Tela BX */
    const IDCONSUIDEMPR = 'CONSULTAIDEMPR'; //Consulta Identificadores Empregador
    const IDCONSUIDTABE = 'CONSULTAIDTABE'; //Consulta Identificadores Empregador
    const IDCONSUIDTRAB = 'CONSULTAIDTRAB'; //Consulta Identificadores Empregador
    /* Operações Download Eventos */
    const IDDOWNLOADID = 'DOWNLOADID'; //Download de Evento por ID
    const IDDOWNLOADREC = 'DOWNLOADREC'; //Download de Evento por Recibo
    
    /* Status dos eventos */
    const IDSTATUS_PENDENTE = 'P';
    const IDSTATUS_ERRO_ENVIO = 'E';
    const IDSTATUS_PROCESSAMENTO = 'M';
    const IDSTATUS_ERRO_PROCESSAMENTO = 'R';
    const IDSTATUS_CONCLUIDO = 'C';
    const IDSTATUS_CONCLUIDO_ADVERTENCIA = 'A';

    /** @var \Doctrine\ORM\EntityManager */
    private $entityManager;
    /** @var \HCM\Util\HCMParameters */
    private $hcmParameters;

    private $nrestruturam;
    
    private $nrorg;
    private $dtmescompetenc;
    
    private $nrEstruEmpregador;
    private $nrParcEmpregador;
    private $tpInscEmpregador;
    private $nrInscEmpregador;
    private $cdNatuJuri;
    
    private $tpAmbiente;
    private $dsAmbiente;
    
    private $nrParcTransmissor;
    private $tpInscTransmissor;
    private $nrInscTransmissor;
    
    private $certFile;
    private $certPass;
    private $certKeyFile;
    private $publicKey;
    private $privateKey;
    
    private $idOperacao;
    
    private $messages;
    
    private $tagNivel;
    private $tagsTrad;
    private $tagsOpc;

    /** @var ESocialLoteLogic */
    private $lote;

    public function __construct($entityManager, $hcmParameters, $nrestruturam, $nrtpambiente) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->nrestruturam = $nrestruturam;
            
        $this->tpAmbiente = $nrtpambiente;
        $this->dsAmbiente = $this->tpAmbiente == 1 ? 'Producao' : 'ProducaoRestrita';
        
        $this->nrorg = $this->hcmParameters->getNrorgTrabalho();
        $this->dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        
        $this->tempEsocialPath = $this->hcmParameters->getTempDirectory() . DIRECTORY_SEPARATOR . "esocial";

        $this->certificatesPath = $this->hcmParameters->getFileServerDirectory();
        $this->tempCertificatesPath = $this->tempEsocialPath . DIRECTORY_SEPARATOR . "certificates" . DIRECTORY_SEPARATOR . $this->hcmParameters->getNrorgTrabalho();

        $this->xmlFilesPath = $this->tempEsocialPath . DIRECTORY_SEPARATOR . "xmlFiles" . DIRECTORY_SEPARATOR . $this->hcmParameters->getNrorgTrabalho();
        
        $this->setDadosEmpregador();

        $this->setDadosTransmissao();
        
        $this->messages = array();
            
        $this->tagsTrad = array();
        $this->tagsOpc = array();
    }
    
    private function setDadosEmpregador() {
        try {
            $estrutura = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->getEstrutura(
                $this->hcmParameters->getNrorgTrabalho(), $this->nrestruturam, $this->hcmParameters->getDtcompetencia()
            );
            
            if ($estrutura) {
                $this->nrEstruEmpregador = $estrutura->getNrestruturam();
                if ($estrutura->historico->getCdcnpjestrut()) {
                    if (strlen($estrutura->historico->getCdcnpjestrut()) == 14) {
                        $this->tpInscEmpregador = '1';
                    } else if (strlen($estrutura->historico->getCdcnpjestrut()) == 11) {
                        $this->tpInscEmpregador = '2';
                    } else {
                        $this->tpInscEmpregador = '';
                    }
                    $this->nrInscEmpregador = $estrutura->historico->getCdcnpjestrut();
                } else {
                    $parcnegocio = $this->entityManager->getRepository(Repositories::PARCNEGOCIO)->findOneBy(
                        array('nrorg'=>$this->hcmParameters->getNrorgTrabalho(), 'nrparcnegocio'=>$estrutura->getNrparcnegocio())
                    );
                    $this->nrParcEmpregador = $parcnegocio->getNrparcnegocio();
                    if ($parcnegocio->getCdtipoinscricao() == 'CNPJ') {
                        $this->tpInscEmpregador = '1';
                    } else if ($parcnegocio->getCdtipoinscricao() == 'CPF') {
                        $this->tpInscEmpregador = '2';
                    } else {
                        $this->tpInscEmpregador = '';
                    }
                    $this->nrInscEmpregador = $parcnegocio->getNrinscricaoparc();
                }
                $this->cdNatuJuri = $estrutura->historico->getCdnatujuri();
            } else {
                throw new \Exception("Não foi encontrada a estrutura " . $this->nrestruturam);
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function setDadosTransmissao() {
        try {
            $esoParametro = $this->entityManager->getRepository(Repositories::ESO_PARAMETRO)->findOneBy(
                array('nrorg'=>$this->hcmParameters->getNrorgTrabalho(), 'nrestruturam'=>$this->nrestruturam)
            );
                
            if (!is_object($esoParametro)) {
                throw new \Exception("Não foi encontrado parâmetro do eSocial para a estrutura " . $this->nrestruturam);
            }
            
            if (!$esoParametro->getNrparcnegocio()) {
                throw new \Exception("Responsável pela transmissão não definido nos parâmetros do eSocial da estrutura " . $this->nrestruturam);
            }
            
            $parcnegocio = $this->entityManager->getRepository(Repositories::PARCNEGOCIO)->findOneBy(
                array('nrorg'=>$this->hcmParameters->getNrorgTrabalho(), 'nrparcnegocio'=>$esoParametro->getNrparcnegocio())
            );
                    
            $this->nrParcTransmissor = $parcnegocio->getNrparcnegocio();
            if ($parcnegocio->getCdtipoinscricao() == 'CNPJ') {
                $this->tpInscTransmissor = '1';
            } else if ($parcnegocio->getCdtipoinscricao() == 'CPF') {
                $this->tpInscTransmissor = '2';
            } else {
                $this->tpInscTransmissor = '';
            }
            
            $this->nrInscTransmissor = $parcnegocio->getNrinscricaoparc();
            
            if (!$esoParametro->getDsarqucert()) {
                throw new \Exception(Messages::HCM09000_ESO_CERTIFICADO_NOT_FOUND_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
            if (!$esoParametro->getDssenhacert()) {
                throw new \Exception(Messages::HCM09000_ESO_SENHA_CERTIFICADO_NOT_FOUND_ERROR, Messages::ERROR_MESSAGE_CODE);
            }

            $this->certFile = $this->certificatesPath . $esoParametro->getDsarqucert();
            $this->certKeyFile = $this->tempCertificatesPath . DIRECTORY_SEPARATOR . "certKey" . $this->nrInscTransmissor . ".pem";
            $this->certPass = $esoParametro->getDssenhacert();
            
            // SOLUÇÃO NOVO FILESERVER	 
            // $this->certFile = Util::buscaArquivoFileServer($this->certFile, '.pfx');	 
            // // $this->certFile é o conteúdo do arquivo buscado	 
            // // comentar linha "$this->certFile = file_get_contents($this->certFile);", pois já retornado o conteúdo do arquivo
            
            // Variável criada para realizar 5 tentativas de acessar o cetificado no FileServer antes de retornar o erro
            $openCertFile = false;
            
            /* Retira possíveis warnings do php ao usar fopen no fileServer */
            error_reporting(E_ALL ^ E_WARNING);
            for ($i=0;$i<5;$i++) {
                if (fopen($this->certFile,"r") == true) {
                    $openCertFile = true;
                    break;
                }
            }
            error_reporting(E_ALL ^ E_NOTICE);
            
            if ($openCertFile) {
                if (file_exists($this->certKeyFile)) {
                    unlink($this->certKeyFile);
                }
                if (!file_exists($this->certKeyFile)) {
                    if (!is_dir($this->tempCertificatesPath)) {
                        mkdir($this->tempCertificatesPath, 0777, true);
                    }
                    // Alteração temporária para ler somente uma vez o arquivo do FileServer e armazenar na variável certFile
                    //$data = file_get_contents($this->certFile);
                    //openssl_pkcs12_read($data, $cert, $this->certPass);
                    $this->certFile = file_get_contents($this->certFile);
                    openssl_pkcs12_read($this->certFile, $cert, $this->certPass);
                    
                    if (is_null($cert)) {
                        throw new \Exception(Messages::HCM09000_ESO_GENERATE_CERT_KEY_ERROR, Messages::ERROR_MESSAGE_CODE);
                    } else {
                        //Dados do certificado
                        $dados = array ();
                        $dados = openssl_x509_parse( openssl_x509_read($cert['cert']) );
                        
                        //Dados mais importantes
                        //$dados['subject']['C']; //País
                        //$dados['subject']['ST']; //Estado
                        //$dados['subject']['L']; //Município
                        //$dados['subject']['CN']; //Razão Social e CNPJ / CPF
                        $validadeCert = date('d/m/Y', $dados['validTo_time_t']);//Validade	
                        $validadeCert = DateUtil::getDataDeString($validadeCert, DateUtil::FORMATO_BRASILEIRO, true);
                        //$dados['extensions']['subjectAltName'];	//Emails Cadastrados separado por ,
                        
                        $dataAtual = date('d/m/Y');
                        $dataAtual = DateUtil::getDataDeString($dataAtual, DateUtil::FORMATO_BRASILEIRO, true);
                        // Verifica se a validade é anterior à data atual
                        if (DateUtil::comparaDatas($validadeCert, $dataAtual)) {
                            throw new \Exception(Messages::HCM09000_ESO_CERTIFICADO_VALIDADE_ERROR, Messages::ERROR_MESSAGE_CODE);
                        } else {
                            $this->publicKey = $this->getPublicKey($cert["cert"]);
                            $this->privateKey = openssl_get_privatekey($cert["pkey"]);
                            
                            file_put_contents($this->certKeyFile, $cert["pkey"] . "\r\n" . $cert["cert"]);
                            chmod($this->certKeyFile, 0777);
                        }
                    }
                }
            } else {
                throw new \Exception(Messages::HCM09000_ESO_CERTIFICADO_NOT_OPEN_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function addMessage($message) {
        $this->messages[] = $message;
    }
    
    public function getMessages() {
        return $this->messages;
    }
    
    private function setTagsTraducao() {
        $tagsTrad = $this->entityManager->getRepository(Repositories::ESO_TAGNAME)->findBy(array('nrorg' => $this->hcmParameters->getNrorgPadrao()));
        
        foreach ($tagsTrad as $tag) {
            $this->tagsTrad[$tag->getDstagname()] = $tag->getDstagtraducao() != '' ?  $tag->getDstagtraducao() : $tag->getDstagname();
            $cdOpcoes = $tag->getDscdopctag() ? explode(';', $tag->getDscdopctag()) : null;
            $nmOpcoes = $tag->getDsnmopctag() ? explode(';', $tag->getDsnmopctag()) : null;
            if($cdOpcoes){
                foreach($cdOpcoes as $index => $cdOpcao){
                    $this->tagsOpc[$tag->getDstagname()][$cdOpcao] = $nmOpcoes[$index];
                }
            }
        }
    }

    private function getPublicKey($key) {
        $data = "";
        $arCert = explode("\n", $key);
        foreach ($arCert as $curData) {
            //remove a tag de inicio e fim do certificado
            if (strncmp($curData, "-----BEGIN CERTIFICATE", 22) != 0 &&
                strncmp($curData, "-----END CERTIFICATE", 20) != 0 ) {
                //carrega o resultado numa string
                $data .= trim($curData);
            }
        }
        return $data;
    }
    
    private function getXmlEvento($dtsName, $dsVersao, $dados) {
        $tpInsc = $this->tpInscEmpregador;
        if (array_search($this->cdNatuJuri, array('1015','1040','1074','1163')) !== false && $this->tpInscEmpregador == '1') {
            $nrInsc = $this->nrInscEmpregador;
        } else {
            $nrInsc = substr($this->nrInscEmpregador,0,8);
        }
        
        /* Monta o array de dados para o xml */
        $dataFactory = new ESocialDataFactory($dtsName, $dsVersao, $tpInsc, $nrInsc);
        $dadosXml = $dataFactory->buildDadosXml($dados, null, false, false, null);
        
        /* Monta o xml do evento*/
        $xmlFactory = new ESocialXmlFactory($dtsName, $dsVersao);
        $xmlEvento = $xmlFactory->buildXml($dadosXml);
        
        /* Salva o xml do evento na pasta temporária */
        //$filePath = $this->saveFile($xmlEvento, $this->xmlFilesPath, $dtsName, 'xml');
        
        /* Assina o xml do evento */
        $xmlEvento = $xmlFactory->assinaXmlEvento($xmlEvento, $this->publicKey, $this->privateKey);
        
        //$retorno = $xmlFactory->validaSchemaXml($xmlEvento);
        
        return $xmlEvento;
    }
    
    private function saveFile($fileContent, $fileDir, $fileName, $fileFormat) {
        if (!is_dir($fileDir)) {
            mkdir($fileDir, 0777, true);
        }
        
        $filePath = $fileDir . DIRECTORY_SEPARATOR . $fileName . "." . $fileFormat;
            
        if (is_file($filePath)) {
            unlink($filePath);
        }
        
        file_put_contents($filePath, $fileContent);
        chmod($filePath, 0777);
        
        return $filePath;
    }

    /**
     * @param $nrparcnegocio
     * @param $nranexo
     * @param $fileContent
     * @param $fileName
     * @param $dsreferencia
     * @return string
     * @throws \Exception
     */
    private function saveXmlOnFileServer($fileContent, $fileName, $dsreferencia, $idGrupo=null) {
        if ($idGrupo == 'I' || $idGrupo == 'T') {
            $dsGrupo = 'Cadastros';
        } else if ($idGrupo == 'N') {
            $dsGrupo = 'NaoPeriodicos';
        } else if ($idGrupo == 'P') {
            $dsGrupo = 'Periodicos';
        } else {
            $dsGrupo = 'Outros';
        }
        
        $filePath = $this->dsAmbiente . "/" . $dsGrupo . "/" . $this->nrInscEmpregador . "/" . $this->idOperacao;
        
        $row = array(
            "DSPATH" => array(
                array(
                    "name"               => $fileName.".xml",
                    "lastModified"       => 1507122512763,
                    "lastModifiedDate"   => DateUtil::getDataAtual()->format(DateUtil::FORMATO_AMERICANO_DATAHORA),
                    "webkitRelativePath" => "",
                    "size"               => strlen($fileContent),
                    "type"               => "application/xml",
                    "b64File"            => "data:application/xml;base64," . (string)base64_encode($fileContent),
                ),
            ),
            "NRORG"          => $this->hcmParameters->getNrorgTrabalho(),
            "NRPARAMETRO"    => "",
            "NRPARCNEGOCIO"  => $this->nrParcEmpregador,
            "NRANEXO"        => null,
            "CDOPERINCLUSAO" => $this->hcmParameters->getCdoperador(),
            "CDOPERADOR_LOG" => $this->hcmParameters->getCdoperador(),
            "DSREFERENCIA"   => $dsreferencia,
            "IDFOTO"         => 'N',
        );
        
        $fileFieldName = "DSPATH";
        
        FileUploader::uploadFilesInRow($row, $fileFieldName, $filePath);
        //$upload = $this->uploadsFileService->setGpeAnexos($row); @TODO Save on GPE_ANEXOS
        
        $filePath = $row['DSPATH'][0]['path'];
        
        return $filePath;
    }
    
    private function consultarIdentificadoresEventos($bxEsocial, $dtsName, $dtsLote, $dsVersao, $dados) {
        /* Monta e assina o xml do evento */
        $xmlEvento = $this->getXmlEvento($dtsName, $dsVersao, $dados);
        
        /* Salva o xml no FileServer */
        $fileName = $bxEsocial->getNrbxesocial() . DateUtil::getDataAtual()->format('YmdHis');
        $xmlPath = $this->saveXmlOnFileServer($xmlEvento, $fileName, $fileName);

        $bxEsocial->setDtenvio(DateUtil::getDataAtual());
        $bxEsocial->setDsdirxmlenv($xmlPath);
        $this->entityManager->persist($bxEsocial);
        
        /* Monta o xml do lote de consulta */
        $xmlFactory = new ESocialXmlFactory($dtsLote, $dsVersao);
        $xmlLote = $xmlFactory->buildXml(array('XMLEVENTO' => $xmlEvento));
        
        /* Realiza a transmissão do xml para o webservice do esocial*/
        $transmissor = new ESocialTransmissor($this->tpAmbiente, $this->idOperacao, $this->certKeyFile);
        $xmlResult = $transmissor->transmitirXml($xmlLote);
        
        /* Trata o retorno da consulta */
        $retorno = $this->trataRetornoConsultaIdentificadoresEventos($bxEsocial, $xmlResult);
        
        return $retorno;
    }
    
    private function trataRetornoConsultaIdentificadoresEventos($bxEsocial, $xmlResult) {
        try {
            $codRetorno = null;
            $idStatus = null;
            $cdErro = null;
            $msg = null;
            
            $xmlObj = simplexml_load_string($xmlResult);

            $codRetorno = (string) $xmlObj->retornoConsultaIdentificadoresEvts->status->cdResposta;
            $msg = (string) $xmlObj->retornoConsultaIdentificadoresEvts->status->descResposta;
            switch($codRetorno) {
                case '201':
                    $idStatus = self::IDSTATUS_CONCLUIDO;
                    $cdErro = null;
                    $this->trataRetornoIdentificadoresEvts($bxEsocial, $xmlObj->retornoConsultaIdentificadoresEvts->retornoIdentificadoresEvts);
                    break;
                case '203':
                    if ($xmlObj->retornoConsultaIdentificadoresEvts->retornoIdentificadoresEvts->qtdeTotEvtsConsulta > 50) {
                        $idStatus = self::IDSTATUS_PROCESSAMENTO;
                    } else {
                        $idStatus = self::IDSTATUS_CONCLUIDO;
                    }
                    $cdErro = null;
                    $this->trataRetornoIdentificadoresEvts($bxEsocial, $xmlObj->retornoConsultaIdentificadoresEvts->retornoIdentificadoresEvts);
                    break;
                case '301':
                case '307':
                case '308':
                case '309':
                case '310':
                    $idStatus = self::IDSTATUS_PENDENTE;
                    $cdErro = (string) $xmlObj->retornoConsultaIdentificadoresEvts->status->cdResposta;
                    break;
                case '402':
                case '403':
                case '404':
                case '405':
                case '406':
                case '407':
                case '409':
                case '410':
                case '411':
                case '417':
                default:
                    $idStatus = self::IDSTATUS_PENDENTE;
                    $cdErro = (string) $xmlObj->retornoConsultaIdentificadoresEvts->status->cdResposta;
                    break;
            }
            
            /* Salva o xml no FileServer */
            $fileName = $bxEsocial->getNrbxesocial() . 'Retorno' . DateUtil::getDataAtual()->format('YmdHis');
            $xmlPath = $this->saveXmlOnFileServer($xmlResult, $fileName, $fileName);
            
            $bxEsocial->setDtresposta(DateUtil::getDataAtual());
            $bxEsocial->setDsdirxmlretenv($xmlPath);
            $bxEsocial->setIdstatus($idStatus);
            $bxEsocial->setCderro($cdErro);
            $bxEsocial->setDsobservacao(substr($msg,0,4000));
            $bxEsocial->setDtultatu(DateUtil::getDataAtual());
            $bxEsocial->setCdoperultatu($this->hcmParameters->getCdoperador());
            $bxEsocial->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($bxEsocial);
            $this->entityManager->flush();

            $this->addMessage($codRetorno . '-' . $msg);
            
            if (is_null($cdErro)) {
                return true;
            } else {
                return false;
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function trataRetornoIdentificadoresEvts($bxEsocial, $xmlObj) {
        try {
            $qtdTotEvts = 0;
            $dhUltimoEvt = null;
            
            $qtdTotEvts = $xmlObj->qtdeTotEvtsConsulta;
            if ($xmlObj->dhUltimoEvtRetornado) {
                $dhUltimoEvt = str_replace('T', ' ', substr((string)$xmlObj->dhUltimoEvtRetornado, 0, 19));
                $dhUltimoEvt = DateUtil::getDataDeString($dhUltimoEvt, 'Y-m-d H:i:s');
            }
            
            foreach ($xmlObj->identificadoresEvts->identificadorEvt as $identificadorEvt) {
                $this->novoEsoEventobxesocial($bxEsocial->getNrbxesocial(), $bxEsocial->getNrevento(), (string)$identificadorEvt->id, (string)$identificadorEvt->nrRec, null);
            }
            
            if ($qtdTotEvts > 50) {
                $this->novoEsoBxesocial(
                    $bxEsocial->getNrestruturam(), $bxEsocial->getNrtpambiente(), $bxEsocial->getNrevento(), $bxEsocial->getDschave(), $dhUltimoEvt, $bxEsocial->getDtfimperiodo(), $bxEsocial->getNrcpftrabalhador(), $bxEsocial->getCdperiodoapuracao()
                );
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function solicitaDownloadEventos($dtsName, $dtsLote, $dsVersao, $eventos) {
        /* Monta e assina o xml do evento */
        $xmlEvento = $this->getXmlEvento($dtsName, $dsVersao, $eventos);
        
        /* Monta o xml do lote de consulta */
        $xmlFactory = new ESocialXmlFactory($dtsLote, $dsVersao);
        $xmlLote = $xmlFactory->buildXml(array('XMLEVENTO' => $xmlEvento));
        
        /* Realiza a transmissão do xml para o webservice do esocial*/
        $transmissor = new ESocialTransmissor($this->tpAmbiente, $this->idOperacao, $this->certKeyFile);
        $xmlResult = $transmissor->transmitirXml($xmlLote);
        
        /* Trata o retorno da consulta */
        $retorno = $this->trataRetornoSolicitacaoDownloadEventos($eventos, $xmlResult);
        
        return $retorno;
    }
    
    private function trataRetornoSolicitacaoDownloadEventos($eventos, $xmlResult) {
        $codRetorno = null;
        $cdErro = null;
        $msg = null;
        
        $xmlObj = simplexml_load_string($xmlResult);

        if ($xmlObj->download->status) {
            $codRetorno = (string) $xmlObj->download->status->cdResposta;
            $msg = (string) $xmlObj->download->status->descResposta;
        } else {
            // Durante os testes quando o esocial não retornoa status é que deu certo
            $codRetorno = '201';
            $msg = 'Download realizado com sucesso';
        }
        
        switch($codRetorno) {
            case '201':
                $cdErro = null;
                break;
            case '301':
            case '307':
            case '308':
            case '309':
            case '310':
            case '402':
            case '403':
            case '404':
            case '405':
            case '406':
            case '407':
            case '409':
            case '410':
            case '411':
            case '417':
                $cdErro = (string) $xmlObj->download->status->cdResposta;
                break;
            default:
                $cdErro = '999';
                $msg = 'Erro do eSocial não identificado';
                break;
        }
        
        $this->addMessage($codRetorno . '-' . $msg);
        
        if (is_null($cdErro)) {
            return true;
        } else {
            return false;
        }
    }
    
    private function retornoSolicDownloadEvtsArquivo($eventos, $xmlObj) {
        $arquivos = array();
        
        foreach ($xmlObj->arquivo as $arquivo) {
            $cdresposta = (string) $arquivo->status->cdResposta;
            $dsresposta = (string) $arquivo->status->descResposta;
            if ($arquivo->evt) {
                $idevento = (string) $arquivo->evt->attributes()->Id;
                $xmlevento = $arquivo->evt->eSocial->asXML();
            } else {
                $idevento = null;
                $xmlevento = null;
            }
            if ($arquivo->rec) {
                $nrrecibo = (string) $arquivo->rec->attributes()->nrRec;
                $xmlrecibo = $arquivo->rec->eSocial->asXML();
            } else {
                $nrrecibo = null;
                $xmlrecibo = null;
            }
            
            $evento = ($eventos[$idevento] ? $eventos[$idevento] : ($eventos[$nrrecibo]));
            
            if (!is_null($evento)) {
                if ($cdresposta == "201") {
                    $eventoBxEsocial = $this->entityManager->getRepository(Repositories::ESO_EVENTOBXESOCIAL)->findOneBy(array('nrorg' => $this->nrorg, 'nreventobxesocial' => $evento['NREVENTOBXESOCIAL']));
                    
                    /* Salva o xml no FileServer */
                    $fileName = $eventoBxEsocial->getNreventobxesocial() . DateUtil::getDataAtual()->format('YmdHis');
                    $xmlPath = $this->saveXmlOnFileServer($xmlevento, $fileName, $fileName);
                    
                    $eventoBxEsocial->setDsdirxmlevento($xmlPath);
                    $eventoBxEsocial->setDtultatu(DateUtil::getDataAtual());
                    $eventoBxEsocial->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $eventoBxEsocial->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                    $this->entityManager->persist($eventoBxEsocial);
                    $this->entityManager->flush();
                } else {
                    $this->addMessage($codRetorno . '-' . $msg . ' (Id:' . $idevento . '/Recibo:' . $nrrecibo . ')');
                }
            }
        }
    }
    
    private function validaConsultaIdentificadores($bxEsocial) {
        if ($bxEsocial->getNrtpambiente() != $this->tpAmbiente) {
            throw new \Exception("Tipo de ambiente do evento diferente do tipo de ambiente parametrizado para a estrutura " . $this->nrestruturam);
        }
        if ($bxEsocial->getIdstatus() == 'C') {
            throw new \Exception("O evento já está concluído. Favor atualizar os dados da tela");
        }
    }
    
    public function getBxesocial($nrbxesocial) {
        $bxEsocial = $this->entityManager->getRepository(Repositories::ESO_BXESOCIAL)
            ->findOneBy(array('nrorg' => $this->nrorg, 'nrbxesocial' => $nrbxesocial));
        
        return $bxEsocial;
    }
    
    public function getEventobxesocial($nreventobxesocial) {
        $eventoBxEsocial = $this->entityManager->getRepository(Repositories::ESO_EVENTOBXESOCIAL)
            ->findOneBy(array('nrorg' => $this->nrorg, 'nreventobxesocial' => $nreventobxesocial));
        
        return $eventoBxEsocial;
    }

    public function novoEsoBxesocial($nrestruturam, $nrtpambiente, $nrevento, $dschave, $dtiniperiodo, $dtfimperiodo, $nrcpftrabalhador, $cdperiodoapuracao, $idstatus='P') {
        try {
            $esoBxesocial = Repositories::ESO_BXESOCIAL;
            
            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoBxesocial $novo */
            $novo = new $esoBxesocial();
            $novo->setNrestruturam($nrestruturam);
            $novo->setNrtpambiente($nrtpambiente);
            $novo->setNrevento($nrevento);
            $novo->setDschave($dschave);
            $novo->setDtiniperiodo($dtiniperiodo);
            $novo->setDtfimperiodo($dtfimperiodo);
            $novo->setNrcpftrabalhador($nrcpftrabalhador);
            $novo->setCdperiodoapuracao($cdperiodoapuracao);
            $novo->setIdstatus($idstatus);
            $novo->setDtinclusao(DateUtil::getDataAtual());
            $novo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $novo->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $novo->setNrbxesocial(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_BXESOCIAL', 1, 12, true));
            
            $this->entityManager->persist($novo);
            
            return $novo;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function novoEsoEventobxesocial($nrbxesocial, $nrevento, $idevento, $nrrecibo, $dsdirxmlevento, $idativo='S') {
        try {
            $esoEventobxesocial = Repositories::ESO_EVENTOBXESOCIAL;
            
            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEventobxesocial $novo */
            $novo = new $esoEventobxesocial();
            $novo->setNrbxesocial($nrbxesocial);
            $novo->setNrevento($nrevento);
            $novo->setIdevento($idevento);
            $novo->setNrrecibo($nrrecibo);
            $novo->setDsdirxmlevento($dsdirxmlevento);
            $novo->setIdativo($idativo);
            $novo->setDtinclusao(DateUtil::getDataAtual());
            $novo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $novo->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $novo->setNreventobxesocial(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_EVENTOBXESOCIAL', 1, 12, true));
            
            $this->entityManager->persist($novo);
            
            return $novo;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function consultarIdentificadoresEventosEmpregador($nrbxesocial) {
        $this->idOperacao = self::IDCONSUIDEMPR;
        
        $bxEsocial = $this->getBxesocial($nrbxesocial);
        
        $this->validaConsultaIdentificadores($bxEsocial);
        
        if ($bxEsocial->getNrevento()) {
            $esoEvento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('nrevento' => $bxEsocial->getNrevento()));
            $filtros['TPEVT'] = (is_object($esoEvento) ? $esoEvento->getNmevento() : null);
        } else {
            $filtros['TPEVT'] = null;
        }
        $filtros['PERAPUR'] = $bxEsocial->getCdperiodoapuracao();
        
        $dtsName = 'ConsultaIdentificadoresEventosEmpregador';
        $dtsLote = 'ConsultarIdentificadoresEventosEmpregador';
        $dsVersao = '1_0_0';
        
        return $this->consultarIdentificadoresEventos($bxEsocial, $dtsName, $dtsLote, $dsVersao, $filtros);
    }
    
    public function consultarIdentificadoresEventosTabela($nrbxesocial) {
        $this->idOperacao = self::IDCONSUIDTABE;
        
        $bxEsocial = $this->getBxesocial($nrbxesocial);
        
        $this->validaConsultaIdentificadores($bxEsocial);
        
        if ($bxEsocial->getNrevento()) {
            $esoEvento = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->findOneBy(array('nrevento' => $bxEsocial->getNrevento()));
            $filtros['TPEVT'] = (is_object($esoEvento) ? $esoEvento->getNmevento() : null);
        } else {
            $filtros['TPEVT'] = null;
        }
        $filtros['CHEVT'] = $bxEsocial->getDschave();
        $filtros['DTINI'] = $bxEsocial->getDtiniperiodo();
        $filtros['DTFIM'] = $bxEsocial->getDtfimperiodo();
        
        $dtsName = 'ConsultaIdentificadoresEventosTabela';
        $dtsLote = 'ConsultarIdentificadoresEventosTabela';
        $dsVersao = '1_0_0';
        
        return $this->consultarIdentificadoresEventos($bxEsocial, $dtsName, $dtsLote, $dsVersao, $filtros);
    }
    
    public function consultarIdentificadoresEventosTrabalhador($nrbxesocial) {
        $this->idOperacao = self::IDCONSUIDTRAB;
        
        $bxEsocial = $this->getBxesocial($nrbxesocial);
        
        $this->validaConsultaIdentificadores($bxEsocial);
        
        $filtros['CPFTRAB'] = $bxEsocial->getNrcpftrabalhador();
        $filtros['DTINI'] = $bxEsocial->getDtiniperiodo();
        $filtros['DTFIM'] = $bxEsocial->getDtfimperiodo();
        
        $dtsName = 'ConsultaIdentificadoresEventosTrabalhador';
        $dtsLote = 'ConsultarIdentificadoresEventosTrabalhador';
        $dsVersao = '1_0_0';
        
        return $this->consultarIdentificadoresEventos($bxEsocial, $dtsName, $dtsLote, $dsVersao, $filtros);
    }
    
    public function downloadEventosPorId($nrbxesocial, $eventos) {
        $this->idOperacao = self::IDDOWNLOADID;
        
        $bxEsocial = $this->getBxesocial($nrbxesocial);
        
        foreach ($eventos as $evento) {
            $ideventos[$evento['IDEVENTO']] = array(
                'NREVENTOBXESOCIAL' => $evento['NREVENTOBXESOCIAL'],
                'IDEVENTO' => $evento['IDEVENTO'],
            );
        }
        
        $dtsName = 'SolicitacaoDownloadEventosPorId';
        $dtsLote = 'SolicitarDownloadEventosPorId';
        $dsVersao = '1_0_0';
        
        return $this->solicitaDownloadEventos($dtsName, $dtsLote, $dsVersao, $ideventos);
    }
    
    public function downloadEventosPorNrRecibo($nrbxesocial, $eventos) {
        $this->idOperacao = self::IDDOWNLOADREC;
        
        $bxEsocial = $this->getBxesocial($nrbxesocial);
        
        foreach ($eventos as $evento) {
            $recibos[$evento['NRRECIBO']] = array(
                'NREVENTOBXESOCIAL' => $evento['NREVENTOBXESOCIAL'],
                'NRRECIBO' => $evento['NRRECIBO'],
            );
        }
        
        $dtsName = 'SolicitacaoDownloadEventosPorNrRecibo';
        $dtsLote = 'SolicitarDownloadEventosPorNrRecibo';
        $dsVersao = '1_0_0';
        
        return $this->solicitaDownloadEventos($dtsName, $dtsLote, $dsVersao, $recibos);
    }
    
    public function setXmlFileToDataSet($xmlString) {
        try {
            $json = $this->parseXmlToJson($xmlString);
            $dataSet = $this->translateESocialJson($json);
            return $dataSet;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function parseXmlToJson($xmlFileContents) {
        try {
    		$xmlFileContents = str_replace(array("\n", "\r", "\t"), '', $xmlFileContents);
    		$xmlFileContents = trim(str_replace('"', "'", $xmlFileContents));
    		$simpleXml = simplexml_load_string($xmlFileContents);
    		$json = json_encode($simpleXml);
    		return $json;
        } catch(\Exception $e) {
            throw $e;
        }
	}
	
	public function translateESocialJson($json) {
        try {
            $rows = array();
            $this->tagNivel = 0;
            $this->setTagsTraducao();
    		$this->translateESocialTag(json_decode($json,true), null, $rows);
    		return $rows;
        } catch(\Exception $e) {
            throw $e;
        }
	}
	
	public function translateESocialTag($tag, $parent, &$rows) {
        try {
            $this->tagNivel++;
	        $id = count($rows) + 1;
    		foreach ($tag as $name => $value) {
    		    if ($name === 'Signature') {
    		        // Não traduz a assinatura do evento
    		        continue;
    		    }
    		    elseif (is_array($value)) {
    		        $rows[] = array(
    		            'ID' => $id,
    		            'PARENT' => $parent,
    		            'NAME' => (is_int($name) ? $name + 1 : (isset($this->tagsTrad[$name]) ? $this->tagsTrad[$name] : $name)),
    		            'VALUE' => null,
    		            '_opened' => true,
    		            'NIVEL' => $this->tagNivel
		            );
    		        $this->translateESocialTag($value, $id, $rows);
    		        $id = count($rows) + 1;
    		    } else {
    		        $rows[] = array(
    		            'ID' => $id,
    		            'PARENT' => $parent,
    		            'NAME' => (isset($this->tagsTrad[$name]) ? $this->tagsTrad[$name] : $name),
    		            'VALUE' => array_key_exists($name, $this->tagsOpc) ? $value.' - '.$this->tagsOpc[$name][$value] : $value,
    		            '_opened' => true,
    		            'NIVEL' => $this->tagNivel
		            );
		            $id = $id + 1;
    		    }
    		}
        } catch(\Exception $e) {
            throw $e;
        }
	}
    
}