<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class NaoPeriodicosItensRepository {

    protected  $connection;

    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function getItensParaEnvioS2190($params) {
        $sql = <<<SQL
    SELECT VH.NRVINCULOM as NRCHAVE,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2190'
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDSTATUS = 'C' /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VH.NRVINCULOM
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   --AND (VM.DTRESCISAOVINC IS NOT NULL AND VM.DTADMISSAOVINC <= :P_DTMESCOMPETENC)   
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2210($params) {
        $sql = <<<SQL
   SELECT VI.NRVITIMA as NRCHAVE,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VI.DTCAT as DTMESCOMPETENC
       FROM SST_VITIMA VI
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2210'
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDSTATUS = 'C' /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VI.NRVITIMA
 WHERE VI.NRORG = :P_NRORG 
   AND (VI.NRVITIMA = :P_NRVITIMA OR :P_NRVITIMA IS NULL)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            //"P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRVITIMA" => isset($params['NRVITIMA']) ? $params['NRVITIMA'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2220($params) {
        $sql = <<<SQL
 SELECT FM.NRFICHAMEDICA as NRCHAVE,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM SST_FICHAMEDICA FM
  JOIN GPE_VINCULOM VM ON FM.NRVINCULOM = VM.NRVINCULOM AND VM.NRORG = FM.NRORG
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2220'
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDSTATUS = 'C' /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VH.NRVINCULOM
 WHERE FM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   --AND (VM.DTRESCISAOVINC IS NOT NULL AND VM.DTADMISSAOVINC <= :P_DTMESCOMPETENC)   
   AND (FM.NRFICHAMEDICA = :P_NRFICHAMEDICA OR :P_NRFICHAMEDICA IS NULL)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRFICHAMEDICA" => isset($params['NRFICHAMEDICA']) ? $params['NRFICHAMEDICA'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2230($params) {
        $sql = <<<SQL
 SELECT ASF.NRALTESITUFUNC as NRCHAVE,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM GPE_ALTESITUFUNC ASF
  JOIN GPE_VINCULOM VM ON VM.NRORG = ASF.NRORG AND ASF.NRVINCULOM = VM.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*Situação Funcional*/
  JOIN GPE_SITUFUNCM SF ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM AND (SF.NRORG = VM.NRORG OR SF.NRORG = 0)
  JOIN GPE_SITUFUNCH SFH ON SFH.NRORG = SF.NRORG
                     AND SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                     AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                               WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                 AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                 AND GPE_SITUFUNCH.NRORG = SFH.NRORG)                                                  
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2230'
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDSTATUS = 'C' /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = ASF.NRALTESITUFUNC
 WHERE ASF.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   --AND (VM.DTRESCISAOVINC IS NOT NULL AND VM.DTADMISSAOVINC <= :P_DTMESCOMPETENC)   
   AND (ASF.NRALTESITUFUNC = :P_NRALTESITUFUNC OR :P_NRALTESITUFUNC IS NULL)
   AND SFH.IDAFASTSITUFUNC = 'S'
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRALTESITUFUNC" => isset($params['NRALTESITUFUNC']) ? $params['NRALTESITUFUNC'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2240($params) {
        $sql = <<<SQL
SELECT MO.NRMOVIMENTACAO as NRCHAVE,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM GPE_MOVIMENTACAO MO    
  JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = MO.NRVINCULOM AND MO.NRORG = VM.NRORG
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN SST_RELESTRUTRISCO RTR ON RTR.NRORG = VH.NRORG AND RTR.NRESTRUTURAM = VH.NRESTRUTGEREN                                                 
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2240'
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDSTATUS = 'C' /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = MO.NRMOVIMENTACAO
 WHERE MO.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM   
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   --AND (VM.DTRESCISAOVINC IS NOT NULL AND VM.DTADMISSAOVINC <= :P_DTMESCOMPETENC)   
   AND (MO.NRMOVIMENTACAO = :P_NRMOVIMENTACAO OR :P_NRMOVIMENTACAO IS NULL)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRMOVIMENTACAO" => isset($params['NRMOVIMENTACAO']) ? $params['NRMOVIMENTACAO'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2241($params) {
        $sql = <<<SQL
SELECT MO.NRMOVIMENTACAO as NRCHAVE,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM GPE_MOVIMENTACAO MO 
  JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = MO.NRVINCULOM AND MO.NRORG = VM.NRORG
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN SST_RELESTRUTRISCO RTR ON RTR.NRORG = VH.NRORG AND RTR.NRESTRUTURAM = VH.NRESTRUTGEREN 
  JOIN SST_FATORRISCO FR ON FR.NRFATORRISCO = RTR.NRFATORRISCO AND FR.NRORG       = RTR.NRORG                                                
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2241'
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDSTATUS = 'C' /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = MO.NRMOVIMENTACAO
 WHERE MO.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM   
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   AND (FR.IDGERAINSAL = 'S' OR FR.IDGERAPERIC = 'S' OR FR.IDGERAAPOSESPEC = 'S')        
   --AND (VM.DTRESCISAOVINC IS NOT NULL AND VM.DTADMISSAOVINC <= :P_DTMESCOMPETENC)   
   AND (MO.NRMOVIMENTACAO = :P_NRMOVIMENTACAO OR :P_NRMOVIMENTACAO IS NULL)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRMOVIMENTACAO" => isset($params['NRMOVIMENTACAO']) ? $params['NRMOVIMENTACAO'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2250($params) {
        $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2250'
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDSTATUS = 'C' /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VH.NRVINCULOM
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   --AND (VM.DTRESCISAOVINC IS NOT NULL AND VM.DTADMISSAOVINC <= :P_DTMESCOMPETENC)   
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2298($params) {
        $sql = <<<SQL
  SELECT VH.NRVINCULOM as NRCHAVE,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2298'
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDSTATUS = 'C' /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VH.NRVINCULOM
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   AND VM.IDTPREINTEGRACAO IS NOT NULL
   --AND (VM.DTRESCISAOVINC IS NOT NULL AND VM.DTADMISSAOVINC <= :P_DTMESCOMPETENC)   
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2299($params) {
        $sql = <<<SQL
 SELECT VH.NRVINCULOM as NRCHAVE,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2299'
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDSTATUS = 'C' /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VH.NRVINCULOM
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   --AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NOT NULL AND VM.DTADMISSAOVINC <= :P_DTMESCOMPETENC)   
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2306($params) {
        $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2306'
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDSTATUS = 'C' /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VH.NRVINCULOM
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   --AND (VM.DTRESCISAOVINC IS NOT NULL AND VM.DTADMISSAOVINC <= :P_DTMESCOMPETENC)
   AND  VM.NRTIPOVINCULOM IN (2,3,10) --Estagiario, Socio, Autonomo
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2399($params) {
        $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2399'
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDSTATUS = 'C' /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VH.NRVINCULOM
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   --AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NOT NULL AND VM.DTADMISSAOVINC <= :P_DTMESCOMPETENC)
   AND  VM.NRTIPOVINCULOM IN (2,3,10) --Estagiario, Socio, Autonomo
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
}