<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 01/09/2016
 * Time: 16:25
 */

namespace HCM\VAB\Controllers;

use HCM\Util\Messages;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\VAB\Services\ValeTransporteService;
use HCM\Util\DateUtil;

class ValeTransporteController extends Crud {

    protected $dataSourceName_getValeTransporte = 'getValeTransporte';
    protected $dataSourceName_setValeTransporte = 'vabLinhavinc_setValeTransporte';
    protected $dataSourceName_getValeTransporteLinhaAtiva = 'getValeTransporteLinhaAtiva';

    protected $parameterBagColumns = array('P_NRORG', 'P_DTMESCOMPETENC', 'P_PROXCOMPETENCIA', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ValeTransporteService $valeTransporteService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->valeTransporteService = $valeTransporteService;
        parent::__construct($dataSourceManager);
    }

    public function getValeTransporte (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        
        $proxCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
        $proxCompetencia->add(new \DateInterval('P1M'));
        $filterData->addCondition('P_PROXCOMPETENCIA','=',$proxCompetencia->format('d/m/Y'));
        
        if ($request->getRoutePath() == '/getValeTransporteLinhaAtiva') {
            $filterData->setDataSourceName($this->dataSourceName_getValeTransporteLinhaAtiva);
        } else {
            $filterData->setDataSourceName($this->dataSourceName_getValeTransporte);
        }
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setValeTransporte (DTO\Request\DataSet $request, DTO\Response $response) {
       $this->dataSourceName = $this->dataSourceName_setValeTransporte;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === true){
                    $row['NRORG'] = $row['NRORGTRAB'];
                    $row['NRLINHAVINC'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'VAB_LINHAVINC', 1, 12, true);
                } 
                
                if( isset($row['NRINTEGTRANSPM']) && !empty($row['NRINTEGTRANSPM']) ){
                    $dtcompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia())->format('Y/m');
                    
                    if(    isset($row['DTFIMLINHAVINC']) 
                        && !empty($row['DTFIMLINHAVINC']) 
                        && DateUtil::getDataDeString($row['DTFIMLINHAVINC'])->format('Y/m') <= $dtcompetencia
                    )
                        $this->valeTransporteService->setIdReserva('S', $row['NRINTEGTRANSPM']);
                    else    
                        $this->valeTransporteService->setIdReserva('N', $row['NRINTEGTRANSPM']);
                }

                if(!array_key_exists('NRINTEGTRANSPM', $row)){
                    $row['NRINTEGTRANSPM'] = null;
                }

                if(!array_key_exists('DSOBSLINHAVINC', $row)){
                    $row['DSOBSLINHAVINC'] = null;
                }
                
                if(!array_key_exists('IDATIVO', $row)){
                    $row['IDATIVO'] = 'S';
                }

                //if( isset($row['NRINTEGTRANSPM']) && !empty($row['NRINTEGTRANSPM']) )
                   // $this->valeTransporteService->retirarValeDaReserva($row['NRINTEGTRANSPM']);
                
            }
                    
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteValeTransporte(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = $this->dataSourceName_setValeTransporte;
            parent::delete($request, $response);

        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}